function [P]=arlike(ar,R,sig2)

// function to calculate the likelihood that a given observation was 
// generated by the autoregressive HMM 

// ar   : matrix of the regression coefficients in each state and for each
// output k (Nx(K*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2(n+1) for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.
// [N,K*(2(n+1)]=size(ar);		
//	     I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I
// ar_k =    I .................................................  I
//           I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I
// K    : Dimension of the system observation vector at time t.
// R	: a matrix n+1xn+1 (if b0=0, AR model) or a matrix 2n+2x2n+2 (if b0=1, ARMA model)
// 	  R is symmetric positive semi-definite
// sig2 : matrix containing the deviation for the regression (K*N)
// N    : number of states

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved
 
[K,N]=size(sig2);
for i=1:N
	P(i)=2/(sqrt(2*%pi*sig2(i)))*exp(-(ar(i,:)*R*ar(i,:)')/(2*sig2(i)));
	if P(i)==0,	// imprecision introduced to avoid problems with precision
		P(i)=1e-300;	// limit of scilab, can lead (rarely) to problems
	end
end



