function [R]=armake(Yk,u,n,b0)

// Creates a matrix n+1xn+1 (if b0=0, AR model): 
// yk=-sum(i=1:n)[ai*y_(k-i)], a0=1;
// or a matrix 2n+1x2n+1 (if b0=1, ARX model)
// yk=-sum(i=1:n)[ai*y_(k-i)]+sum(j=0:n)[bj*y_(k-j)], a0=b0=1
// from the observation segment of length Ts of kth system output : Oout(k,1:Ts), K=> k >=1
// Yk 	: Oout(k,(t-1)*Ts+1:Ts*t) (kth output observation segment vector), t=1,2,..,T
// n=na=nb 
// na   : number of regression coefficients for the output signal
// nb 	: number of regression coefficients (nb<=na) for the input signal, b0=0 if no input signal is used
// u 	: Oin(1:Ts) (Input segment vector). u=0 if no input signal is used
// b0	: b0=0 (u=0), AR model,  b0=1, ARX model

// R	: a matrix (K*(n+1xn+1)) (if b0=0, AR model) or a matrix (K*2(n+1)x2(n+1)) (if b0=1, ARX model)
// 	  R is symmetric positive semi-definite

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


U=[];
Ts=size(Yk,2); // Input and output segment length.				
for i=0:n			//	   I yn+1 yn+2   . . . yTs   I
	U=[U;Yk(n-i+1:Ts-i)];	// U(Yk) = I ..................      I
end				//	   I y1    y2    . . . yTs-n I

// If na~=nb, then only the values of u from t=na-nb to Ts may be taken into account

if b0~=0,
   for j=0:n			//	     I yn+1     yn+2     . . .  yTs   I
	U=[U;u(n-j+1:Ts-j)];	// U(u,Yk) = I ..........................     I
   end				//	     I y1       y2       . . .  yTs-n I
end				//	     I un+1     un+2     . . .  uTs   I
				//  	     I ............................   I
				//	     I u1       u2       . . .  uTs-n I

R=U*U';				


