function [LP,P]=armulik1(ar,sig2,Ts,b0,Oout,Oin)


// function to calculate the Normalized logarithm of the likelihood that the 
// autoregressive HMM has created the observation sequence Oout. 
// Only valid for one state models and non correlated output components.

// Ts   : number of observations at least represented by one state.
// b0	: b0=0 (u=0), AR model,  b0=1, ARX model
// as   : initial state distribution (Pinit).
// a    : state transition matrix (here not calles A as usually).
// ar   : matrix of the regression coefficients in each state and for each
//        output k (Nx(K*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2(n+1) for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.
//        [N,K*(2(n+1)]=size(ar);		
//	     I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I
// ar(:,k) = I .................................................  I
//           I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I
// K    : Dimension of the system observation vector at time t.
// n    : number of regression coefficints for the input or output signal (n=na=nb).
// N    : number of states
// sig2	: Squared SIGMA matrix (K,N);
// Oout : output observation matrix of dimension (Kx(TxM)), system output is a vector of size K.
// Oin  : input observation vector de dimension 1x(TxM), system input is scalar.
// P    : Likelihood
// LP	: Normalized logarithm of the likelihood (LP/T)

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[K,T]=size(Oout);
[N,Np1]=size(ar);
Ng=Np1/K;
n=(Ng/2)-1;
T=fix(T/Ts);
LP=0;
for t=1:T
    for k=1:K,
        R=armake(Oout(k,(t-1)*Ts+1:Ts*t),Oin((t-1)*Ts+1:Ts*t),n,b0);
        LP=LP+log(2/sqrt(2*%pi*sig2(k,:)))-ar(:,(k-1)*Ng+1:k*Ng)*R*ar(:,(k-1)*Ng+1:k*Ng)'/(2*sig2);
    end
end
P=exp(LP);
LP=LP/T; // Normalized log (likelihood)






