function [LP,P]=armuliks(Pinit,A,ar,sig2,Ts,b0,Oout,Oin)

// calculation of the Normalized log(likelihood) that the reestimated model has generated the
// output observation       
// This function takes the underflow problem into account 

// Ts   : number of observations at least represented by one state.
// b0	: b0=0 (u=0), AR model,  b0=1, ARX model
// Pinit: initial state distribution (1*N).
// A    : state transition matrix (N*N)
// N    : number of states
// ar   : matrix of the regression coefficients in each state and for each
// output k (Nx(K*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2(n+1) for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.
// [N,K*(2(n+1)]=size(ar);		
//	     I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I
// ar(:,k) = I .................................................  I
//           I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I
// K    : Dimension of the system observation vector at time t.
// sig2	: Squared SIGMA matrix (K,N);
// Oout : output observation matrix of dimension (K*(T*Ts)), system output is a vector of size K.
// Oin  : input observation vector de dimension (1*(T*Ts)), system input is scalar.
// LP	: Normalized log of Likelihood of HMM (log(P(O|model)/T)

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[K,T]=size(Oout);    // calculation of some variables from the function input
[N,N]=size(A);
[N,Np1]=size(ar);
Ng=Np1/K;
// n    : number of regression coefficints for the input or output signal (n=na=nb).
n=(Ng/2)-1;
T=fix(T/Ts);
s=zeros(1,T);        // matrix with the scaling values
Alpha=zeros(N,T);    // Alpha : array to store the forward prob.
Beta=zeros(N,T);     // Beta :array to store the backward prob.

// calculation of Alpha for t=1
    
P=ones(N,1);
for k=1:K            // calculation of the likelihood
    R=armake(Oout(k,1:Ts),Oin(1:Ts),n,b0);
    P=P.*arlike(ar(:,(k-1)*Ng+1:k*Ng),R,sig2(k,:));
end
Alpha(:,1)=Pinit'.*P;
    if and(Alpha(:,1)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
	abort;
    end
s(1)=1/sum(Alpha(:,1));            
Alpha(:,1)=s(1)*Alpha(:,1);    // scaling 

// calculation of Alpha for t=2,...T

for t=2:T
    P=ones(N,1);
    for k=1:K                    // calculation of the likelihood
        R=armake(Oout(k,(t-1)*Ts+1:Ts*t),Oin((t-1)*Ts+1:Ts*t),n,b0);
        P=P.*arlike(ar(:,(k-1)*Ng+1:k*Ng),R,sig2(k,:));
    end
    Alpha(:,t)=((Alpha(:,t-1)'*A).*P')';
    if and(Alpha(:,t)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
	abort;
    end
    s(t)=1/sum(Alpha(:,t));
    Alpha(:,t)=s(t)*Alpha(:,t);   

end

// calculation of the likelihood that the given model has produced 
// the observation sequence

if or(s==0) then
  LP= -%inf 
   P=exp(LP);
else
LP=-sum(log(s))/T; // normalized likelihood	
 P=exp(LP);	
end

        








