function [Obs,Xs]=cgenlrb(Pinit,A,Me,Seg,C)

// This function is used to test the function cannlr.sci
// function to generate observation matrices for left to right model
// reestimation test. The several observations are concatenated with their
// length in between them.
// The vector Tv contains the observation lengths for the several observations
// The observations are modeled by a mixture of Gaussians
// To obtain a HMM model, execute the function cinhmm.sci
// with the argument "lr" set to 2 before using this function

// Pinit    : Initial state distribution vector (1*N)
// A        : State stochastic matrix (N*N)
// Seg      : Observation Covariance list Seg=list(1:M)Segi
// Segi     : Observation covarianc list Segi=list(1:N)Segsi
// Segsi    : Covariance Matrix (K*K) in state si
// Me       : Observation mean list Me=list(1:M)Mi
// Mi       : Matrices  Mi (K*N)of mean vectors Msi
// Msi      : Observation Mean vectors (K*1) in state si
// C        : Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*1)  : Case of Single Gaussian
// C (N*M)  : Case of M mixtures
// Obs      : Obs is a L dimonsional list of martrices (KxT): [O1 O2 .. OT]';
//            where Oi are the continuous observation vectors (Kx1);
//      size(Obs): (1,L+Ts1+Ts2+...+TsL)
// Xs       : State sequences vector :the several state sequences X are
//      concatenated with their length Ts inbetween them
//      size(Xs): (1,L+Ts1+Ts2+...+TsL)
// X        : State sequence vector (1*T)
// M=1      : Number of Gaussian observation mixtures (scalar)
// K        : Observation mean vector dimension
// N        : Number of states in HMM (scalar)
// T        : Observation sequence length (scalar)

// Copyright (c) 2000 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=5,
    printf("Wrong number of input arguments in function CGENOM. Function call is:\n");
    printf("[ Obs,Xs ] = cgenlrb( Pinit , A , Me , Seg , C )\n");
    printf("Program termination\n");
    abort;
end
if lhs~=2,
    printf("Wrong number of output arguments. Continue ? ( y/n )\n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y'),
        printf("Program termination\n");
        abort;
    end
end
[N,N]=size(A);
M=size(Me);
K=size(Me(1),1);
Obs=zeros(K,1);

// check of the consistency of the model parameters
if ~cchkparm(Pinit,A,Me,Seg,C,Obs),     // check if the sizes of A,Pinit, Me, Seg are consitent
    printf("Program termination\n");
    abort;
end
    check=%f
    while ~check,
        L=500;
        L=evstr(x_dialog(['NUMBER OF OBSERVATION SEQUENCES (L)';
        'You have chosen Left-to-Right DHMM.';
        'You may introduce the number of the several observation sequences';
        'Example: L=3 for 3 observation sequences, of length T=200, 400, and 300 respectively.';
        'Please click on OK to continue, otherwise Cancel to abort'],table(L)));
        if L~=[],
        check=%t;
        else
            yorn();
        end
    end
SL=x_message(['Are the multiple observation sequences have the same length?'],['Yes','No']);
            if SL == 1 then
//=================

    check=%f
    while ~check,
        Ts=x_dialog(['OBSERVATION SEQUENCE LENGTHS VECTOR Tv (1,L)';
        'You may introduce the observation sequence length. Then the several observations';
        'are concatenated with their length inbetween them to obtain the observations vector Obs';
        'Example: for L=3 observation sequences, of length T=200, Tv=[200 200 200]';
        'In this case Obs=[200 O1 O2 .. O200 200 O1 O2 .. O200 200 O1 O2 .. O200]';
        'Oi are the continuous observation vector';
        'Please click on OK to continue, otherwise Cancel to abort'],'50');
        if Ts~=[],
        check=%t;
        else
            yorn();
        end
    end
    Tv=evstr(Ts)*ones(1,L);
//=================
            elseif SL==2 then
//=================

    check=%f
    while ~check,
        Tv=5*ones(1,L);// for demostration
        Tv=evstr(x_dialog(['OBSERVATION SEQUENCE LENGTHS VECTOR Tv (1,L)';
        'You may introduce the observation lengths for the several observation sequences';
        'Then the several observations are concatenated with their length inbetween them';
        'to obtain the observations vector Obs';
        'Example: for L=3 observation sequences, of length T=200, 400, and 300 respectively.';
        'Then  Tv=[200 400 300]';
        'In this case Obs=[200 O1 O2 .. O200 400 O1 O2 .. O400 300 O1 O2 .. O300]';
        'Oi are the continuous observation vector';
        'Please click on OK to continue, otherwise Cancel to abort'],table(Tv)));
        if Tv~=[],
        check=%t;
        else
            yorn();
        end
    end
//=================
            end

Obs=list();
Xs=list();

for l=1:L // L is the sequence number
    T=Tv(l);
    O=zeros(K,T);
    X=zeros(1,T);
    // Observations and states generation
    // Calculate cumulated probability for A, Pinit and C
    CumA=A;
    CumPi=Pinit;
    for i=1:N-1
        CumA(:,i+1)=CumA(:,i)+A(:,i+1);
        CumPi(i+1)=CumPi(i)+Pinit(i+1);
    end
    CumC=C;
    for m=1:M-1
        CumC(:,m+1)=CumC(:,m)+C(:,m+1);
    end
    // choose initial state
    rand('uniform');
    x=rand(1);
    i=1;
    while x>CumPi(i), i=i+1; end
    for t=1:T
        X(t)=i;
        // Choose mixture randomly
        rand('uniform');
        x=rand(1);
        m=1;
        while x>CumC(i,m), m=m+1; end
        // Calculate random observation with mixture m
        Met=Me(m)(:,i);
        U=Seg(m)(i);
        [EigVal,Q]=bdiag(U);
        rand('normal');
        y=rand(K,1);
        y=sqrt(EigVal)*y;
        O(:,t)=Q*y+Met;
        // Choose next state
        rand('uniform');
        x=rand(1);
        j=1;
        while x>CumA(i,j), j=j+1; end
        i=j;
    end
    Obs(l)=O;
    Xs(l)=X;
end
