function [Pinit,A,Me,Seg,C,M,K,N] = cinhmm(mod,lr)

// Two examples for a Multivariate single, M=1, C=ones(N,M)
// or mixture, M > 1, C=ones(N,M) Continuous Observations HMM (CHMM) initialisation
// returns the parameters of the mod-th model (mod=1 or 2)
// Two model structures may be generated : 
// mod=1 for one mixture (M=1), 2 states (N=2), 2 components in the observation vector (K=2)
// mod=2 for two mixtures (M=2), 2 states (N=2), 2 components in the observation vector (K=2)
// lr=1 : All transitions between states are permitted
// lr=2 : Left-to-Right model

// Pinit 	: Initial state distribution vector (1*N)
// A     	: State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*1) 	: Case of Single Gaussian
// C (N*M)	: Case of M mixtures
// M		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// mod 		: Model index (i=1 or 2)
// mod=1	: N=2, M=1, K=2  
// mod=2	: N=2, M=2, K=2  
// lr		: Model type index 
// lr=1 	: All transitions between states are permitted
// lr=2		: Left-to-Right model

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved
 
[lhs,rhs]=argn(0);
if rhs~=2
	mod=1;lr=1;
	printf("No input arguments specified. Inputs set to defaults mod=1, lr=1.\n");
	printf("Continue? (y/n) then return\n");
	ans=read(%io(1),1,1,'(a)');
	if ~((ans=='y')|(ans=='Y')),
		printf("Program termination\n");
		abort;
	end
end
if (mod<1)|(mod>2) 
	if (lr<1)|(lr>2) then
		mod=1;lr=1;
		printf("First input argument mod must be equal to 1 or 2. This input set to default mod=1.\n");
		printf("Second input argument lr must be equal to 1 or 2. This input set to default lr=1.\n");
		printf("Continue? (y/n) then return\n");
		ans=read(%io(1),1,1,'(a)');
		if ~((ans=='y')|(ans=='Y')),
			printf("Program termination\n");
			abort;
		end
	else
		mod=1;
		printf("First input argument mod must be equal to 1 or 2. This input set to default mod=1.\n");
		printf("Continue? (y/n) then return\n");
		ans=read(%io(1),1,1,'(a)');
		if ~((ans=='y')|(ans=='Y')),
			printf("Program termination\n");
			abort;
		end
	end
end
if (lr<1)|(lr>2) 
	if (mod<1)|(mod>2) then
		mod=1;lr=1;
		printf("First input argument mod must be equal to 1 or 2. This input set to default mod=1.\n");
		printf("Second input argument lr must be equal to 1 or 2. This input set to default lr=1.\n");
		printf("Continue? (y/n) then return\n");
		ans=read(%io(1),1,1,'(a)');
		if ~((ans=='y')|(ans=='Y')),
			printf("Program termination\n");
			abort;
		end
	else
		lr=1;
		printf("Second input argument lr must be equal to 1 or 2 . This input set to default mod=1.\n");
		printf("Continue? (y/n) then return\n");
		ans=read(%io(1),1,1,'(a)');
		if ~((ans=='y')|(ans=='Y')),
			printf("Program termination\n");
			abort;
		end
	end
end
if lhs~=8
	printf("Wrong number of output variables\n");
	printf("Continue? (y/n)\n");
	ans=read(%io(1),1,1,'(a)');
	if ~((ans=='y')|(ans=='Y')),
		printf("Program termination\n");
		abort;
	end
end
select mod
case 1,	
		K=2;			// Observation vector dimension (scalar)
		N=2;			// Number of states in HMM1 (scalar)
		M=1;			// Number of Gaussian observation mixtures (scalar)
					
					// i : state index
					// m : mixture index
		if lr==1 then		
		   Pinit =[0.8 0.2];	// Initial probability vector (1*N matrix)
		   A=[0.5 0.5		// Transition probability matrix (N*N matrix)
		      0.5 0.5];
		elseif lr==2 then
		   Pinit =[1.0 0.0];	// Initial probability vector (1*N matrix)
		   A=[0.5 0.5		// Transition probability matrix (N*N matrix)
		      0.0 1.0];
        	end  
		
                Me =list();		// Me=list(m)(Mi)
                M1=[-10.0;-1.0];		// Observation mean vector in state i (Mi(i=1)=MeanO1(K,1));
                M2=[-6.0 ;-3.4];		// Observation mean vector in state i (Mi(i=2)=MeanO2(K,1));
		m=1;			// Gaussian observation mixture index (m=M=1) 
                Me(1)=[M1 M2];          // Observation mean vectors in melange m and state i (Me=list(m)(i));
		Segi=list();		// Segi=list(i)(SegOi)
		Seg=list();		// Seg=list(m)(Segi)
//		Segi(1)=[5 2;2 5];// Observation covariance matrix in state i (Segi(i=1)=CovO1(K,K)); 
		Segi(1)=[5 0;0 5];// Observation covariance matrix in state i (Segi(i=1)=CovO1(K,K));
		Segi(2)=Segi(1);	// Observation covariance matrix in state i (Segi(i=2)=CovO2(K,K)); 
		Seg(1)=Segi;		// Observation covariance matrices in melange m and state i (Seg=list(m)(i));
		C=ones(N,M);			// Mixture coefficient matrix (N*M) matrix independent of k)
		
case 2,	

M=2;
K=2;
N=2;
		if lr==1 then
		   Pinit=[0.8 0.2];	// Initial probability vector (1*N matrix)
		   A=[0.7 0.3;		// Transition probability matrix (N*N matrix)
		      0.2 0.8];
		elseif lr==2 then     
 		   Pinit =[1.0 0.0];	// Initial probability vector (1*N matrix)
		   A=[0.7 0.3
		      0.0 1.0];
		end        
		Me =list();		// Me=list(m)(Mi)
		//     k=1  2  
		M11=[5 ; 6];		// m=1, i=1
		M12=[6 ; 7];		// m=1, i=2
		M21=[3 ; 4];		// m=2, i=1
		M22=[3 ; 6];		// m=2, i=2

		
		Me(1)=[M11 M12];	// m=1
		Me(2)=[M21 M22];	// m=2
	
		Segi1= list()		// Segi1=list(i)(SegOi)
		Segi2= list()		// Segi1=list(i)(SegOi)
		Seg=list();		// Seg=list(m)(Segi)
		Segi1(1)=[5 2;2 5];	// Observation covariance matrix in state i=1 and Mixture m=1; 
		Segi1(2)=[3 1;1 3];	// Observation covariance matrix in state i=2 and Mixture m=1;
		Segi2(1)=[4 3;3 4];	// Observation covariance matrix in state i=1 and Mixture m=2;  
		Segi2(2)=[1 0.5;0.5 1];	// Observation covariance matrix in state i=2 and Mixture m=2; 

		Seg(1)=Segi1;	// Observation covariance matrices in melange m=1 
				// for states i=1 and i=2;
		Seg(2)=Segi2;  	// Observation covariance matrices in melange m=2 
				// for states i=1 and i=2;

		C=[0.5 0.5;
		   0.5 0.5];

end









