function [LP,P]=clikelih(Pinit,A,Me,Seg,C,O)

// This function calculates the probability density function that a given model has 
// produced the given single gaussian observation sequence. The calculation is done 
// with the forward probabilities of the Baum-Welch algorithm and 
// takes the underflow problem into account
// L.R. Rabiner : A Tutorial on HMM and selected applications in speech recognition, IEEE 1989

// O     	: Given observation sequence matrix (K*T)
// N		: Number of states in HMM (scalar)
// M		: Number of Gaussian observation mixtures (scalar)
// T     	: Given Observation sequence length (scalar)
// Pinit 	: Initial state distribution vector (1*N)
// LP		: Log of the Normalized Likelihood (log(P(O))/T)
// P		: Normalized Likelihood
// A     	: Given State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// m=1,.., M 	: mixture index;i=1,..,N : state index
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*1) 	: Case of Single Gaussian
// C (N*M)	: Case of M mixtures
// i 		: state index
// m 		: mixture index

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


c=(2*%pi)^(-K/2);
[K,T]=size(O);		// K : number of observations per state T : observation length
[N,cA]=size(A);		// N : number of states
s=zeros(1,T);		// s : scaling factor
Alpha=zeros(N,T);	// Alpha : array to store the forward prob.
Beta=zeros(N,T);	// Beta :array to store the backward prob.
Ro=list();              // Ro : list to store the scaled forward prob for a certain m
M=size(Me);             // M : number of mixtures

if size(C,2)==1 then 

// GaussMat is a matrix (N*T) of bi(Ot) 
 
	[GaussMat]=gengauss(Me(1),Seg(1),O);
else

// GaussMatm 	: A list of dimension t of matrices (N,M),generated by gengauss.sci

	[GaussMatm,GaussMat]=gengausm(C,Me,Seg,O);
end			
// calculation of Alpha and Ro at t=1 and for i=1,..,N 
//t=1;
Alpha(:,1)=Pinit'.*GaussMat(:,1);
if and(Alpha(:,1)==0),
  printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero. We add epsilon\n");
  Alpha=Alpha+%eps*ones(N,T);
end
s(1)=1/sum(Alpha(:,1));
Alpha(:,1)=s(1)*Alpha(:,1);

// calculation of Alpha and R for t=1,..,T and for i=1,..,N 
for t=2:T
	Alpha(:,t)=(A'*Alpha(:,t-1)).*GaussMat(:,t);
	if and(Alpha(:,t)==0),
	  printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero. We add epsilon\n");
	  Alpha=Alpha+%eps*ones(N,T);
	end
	s(t)=1/sum(Alpha(:,t));
	Alpha(:,t)=s(t)*Alpha(:,t);
end
// Calculation of P(O\Lambda)		// Probability that the "old" model has generated 
					// the observation sequence
		
//	LP=-sum(log(s));
// Calculation of P(O|model)
if or(s==0) then
  LP= -%inf
  P=exp(LP);
else
  LP=-sum(log(s));
  P=exp(LP);
  LP=LP/T; // normalized likelihood		
end


