function [LP,Pinit,A,Me,Seg,C]=clrrecmg(Pinit,A,Me,Seg,C,Obs)

// This function calculates the Forward and Backward Probabilities
// and then reestimates the Left-to-right CHMM. The conditonal probability of
// a (K) dimonsional observation is considered to be a single or a mixture of M
// gaussian densities with mean vectors (Mi) and covariance matrices
// (Segi).
// A multiple observation sequences are necessary for left to right models since
// only these models need a training with different observation sequences.
// ( Pinit(1)=1, Pinit(i)=0, i~=1 )
// ( A(i,j)=0,j>i+delta ).
// If there are problems with divisions by zero which might be caused by a bad initial guess of the model parameters, a better initial guess often solves this.
// The reestimation formulas are obtained by Liporace Algorithm.
// This function takes the underflow problem into account.
// LP : log of mean probability that the new model has generated the observation sequences.
// see Rabiner et al. : A model-based connected-digit recognition system
// using either HMMs or templates, L.R.Rabiner et al., IEEE Computer Speech and
// Language 1986
// and Rabiner : A Tutorial on HMM and selected applications in speech recognition,
// L.R.Rabiner, IEEE 1989

// Obs      : The observation list obs consists of the different observation
//            matrices (sequences) of dimension (KxT), where K is the
//            dimension of the observation vectot at time t and T is
//            the sequence length.
// Pinit    : Initial state distribution vector (1*N)
// A        : State stochastic matrix (N*N)
// Seg      : Observation Covariance list Seg=list(1:M)Segi
// Segi     : Observation covarianc list Segi=list(1:N)Segsi
// Segsi    : Covariance Matrix (K*K) in state si
// Me       : Observation mean list Me=list(1:M)Mi
// Mi       : Matrices  Mi (K*N)of mean vectors Msi
// Msi      : Observation Mean vectors (K*1) in state si
// C        : Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*M)  : Case of Single Gaussian
// C (N*M)  : Case of M mixtures
// M        : Number of Gaussian observation mixtures (scalar)
// K        : Observation mean vector dimension
// N        : Number of states in HMM (scalar)
// T        : Observation sequence length (scalar)
// LP       : Normalized log of Likelihood of HMM (log(P(O|model)/T)

// Copyright (c) 2000 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


L=size(Obs);    // L : dimension of the observation list (number
                //of observation sequences)

[N,N]=size(A);      // N : number of states

// check of Pinit

if Pinit(1)~=1,
    printf('It has to be Pinit(0)=1 for left to right models\n');
    printf('Program termination\n');
    abort;
end
for i=2:N
    if Pinit(i)~=0,
        printf('It has to be Pinit(i)=0, i>1 for left to right models\n');
        printf('Program termination\n');
        abort;
    end
end

SumDAl=zeros(N,1);  // Denominator to reestimate A for a single observation
SumDCl=SumDAl;      // Denominator to reestimate C for a single observation
DC=SumDAl;      // Denominator to reestimate C for the set of observations
DA=SumDAl;      // Denominator to reestimate A for the set of observations
Sumrobetal=zeros(N,M);  // Denomiator to reestimate Seg and Me for a single observation
Sumrobetall=Sumrobetal; // Denomiator to reestimate Seg and Me for the set of observations
NAl=zeros(N,N);     // Nominator to reestimate A for a single observation sequence
NA=NAl;         // Nominator to reestimate A for the set of observations sequence
NCl=zeros(N,M);     // Nominator to reestimate C for a single observation sequence
NC=NCl;         // Nominator to reestimate C for the set of observations sequence
NMel=list();        // Nominator to reestimate Me for a single observation sequence in mixture m
Mel=list();     // Reestimated Me for a single observation sequence in mixture m
NMeall=list();      // Nominator to reestimate Me for the set of observations sequence in mixture m
NSegl=list();       // Nominator to reestimate Seg for a single observation
buf1=list();
Nil=list();
buf2=list();
buf1(1)=zeros(K,K);
NSegil=zeros(K,K);  // Nominator to reestimate Seg for a single observation sequence in state i
for i=1:N, buf1(i)=NSegil;end;
for m=1:M, NSegl(m)=buf1;end;
NSegall=NSegl;      // Nominator to reestimate Seg for the set of observations sequence
NC=zeros(N,M);
c=(2*%pi)^(-K/2);
M=size(Me);             // M : number of mixtures
LPl=0;
LP=0;
// GaussMatm    : A list of dimension t of matrices (N,M),generated by gengauss.sci
//        calculates for t=1,..,T  bim(Ot) in state i and mixture m
// GaussMat     : A list of dimension t of vectors (N,1), the vectors components are bj(Ot)

        for m=1:M
            NMel(m)=zeros(K,N); // Nominator to reestimate the mean matrix Me for a single observation
            Mel(m)=zeros(K,N);  // mean matrix Me for a single observation
            NMeall(m)=zeros(K,N);

        end

for l=1:L,
    O=Obs(l);   // observation matrix taken from the observation
                //list
    [K,T]=size(O);
    l=l+T+1;
    printf('Iteration with the observation of length T=%d\n',T);buf=list();
    buf2(1)=zeros(1,T);
    for i=1:N, buf2(i)=Nil; end;
    for m=1:M, Nil(m)=buf2;end;
    s=zeros(1,T);       // s : scaling factor
    Alpha=zeros(N,T);   // Alpha : array to store the forward prob.
    Beta=zeros(N,T);    // Beta :array to store the backward prob.
    Ro=list();              // Ro : list to store the scaled forward prob for a certain m
    [GaussMatm,GaussMat]=gengausm(C,Me,Seg,O);

    for m=1:M
        Ro(m)=zeros(N,T);
    end

    // calculation of Alpha and Ro at t=1 and for i=1,..,N

    t=1;
    Alpha(:,1)=Pinit'.*GaussMat(:,1);
    s(1)=1/sum(Alpha(:,1));
    Alpha(:,1)=s(1)*Alpha(:,1);

    if and(Alpha(:,1)==0),
        printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
        abort;
    end
    for m=1:M
        Ro(m)(:,1)=s(1)*Pinit'.*GaussMatm(m)(:,1);
    end

// calculation of Alpha and R for t=1,..,T and for i=1,..,N
    for t=2:T
        Alpha(:,t)=(A'*Alpha(:,t-1)).*GaussMat(:,t);
        s(t)=1/sum(Alpha(:,t));
        Alpha(:,t)=s(t)*Alpha(:,t);

        if and(Alpha(:,t)==0),
            printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n");
            abort;
        end
        for m=1:M
            Ro(m)(:,t)=s(t)*(A'*Alpha(:,t-1)).*GaussMatm(m)(:,t);
        end
    end
// Calculation of P(O\Lambda)       // Probability that the "old" model has generated
                                // the observation sequence

//  LPl=-sum(log(s));
// Calculation of P(O\Lambda)
    if or(s==0) then
        LPl= -%inf
    else
        LPl=-sum(log(s))/T; // normalized likelihood
        LP=LP+LPl;
    end
// Calculation of Beta at t=T for i=1,..,N
Beta(:,T)=s(T)*ones(N,1);
// calculation of Beta for t=T-1,..,1 and for i=1,..,N

    for k=1:T-1
        t=T-k;
        Beta(:,t)=s(t)*A*(GaussMat(:,t+1).*Beta(:,t+1));
    end
// Reestimation of model parameters
// Re-estimation of the model parameters
// the scales variables Alphas and Betas contain already a division by
// the probability that the actual observation was generated by the
// model so that Rabiner (109) is fullfilled automatically

// Calculation of the Denomiators for the reestimation
    Is=ones(1,T)./s;
    SumDCl=(Alpha.*Beta)*Is';
    SumDAl=SumDCl-Alpha(:,T).*Beta(:,T)/s(T);
    if min(abs(SumDCl))==0,
        printf("Warning : Too small values in the denominator in the reestimation of Me or Seg).\nThis leads to a division by zero.\n");
        printf("Program termination\n");
        abort;
    end
        DC=DC+SumDCl;
        DA=DA+SumDAl;
// Calculation of the nominator for the reestimation of the state
// transition matrix (A)

for j=1:N
    NAl(:,j)=A(:,j).*(Alpha(:,1:T-1)*(GaussMat(j,2:T).*Beta(j,2:T))');
end
        NA=NA+NAl;
// Calculation of the deniminator for the reestimation of the covariance matrix (Seg) for sequence l
    Sumrobetal=zeros(N,M);
    for m=1:M
        Sumrobetal(:,m)=(Ro(m).*Beta)*Is';
    end
    if min(abs(Sumrobetal))==0,
        printf("Warning : Too small values in the denominator in the reestimation of Me and Seg ).\nThis leads to a division by zero.\n");
        printf("Program termination\n");
        abort;
    end
    Sumrobetall=Sumrobetall+Sumrobetal;
// Calculation of the nominator for the reestimation of the covariance matrix (NSegl) for sequence l
// Calculation of the nominator for the reestimation of the mean matrix (NMel) for sequence l

    for i=1:N
        for m=1:M
            Nil(m)(i)=Ro(m)(i,:).*Beta(i,:).*Is;
            NMel(m)(:,i)=O*Nil(m)(i)';
            Mel(m)(:,i)=NMel(m)(:,i)/Sumrobetal(i,m);
        end
    end
        for m=1:M
            NMeall(m)=NMeall(m)+NMel(m);
        end
    for i=1:N
        for m=1:M
            Mil=(O-Mel(m)(:,i)*ones(1,T));
            NSegil=((ones(K,1)*Nil(m)(i)).*Mil)*Mil';
            NSegl(m)(i)=NSegil;
        end
    end
        for i=1:N
            for m=1:M
                NSegall(m)(i)=NSegall(m)(i)+NSegl(m)(i);
            end
    end
// Calculation of the nominator for the reestimation of the gain matrix (C)

    NC=NC+Sumrobetal;
end
    A=NA./(DA*ones(1,N));
    C=NC./(DC*ones(1,M));
    for i=1:N
        for m=1:M
            Me(m)(:,i)=NMeall(m)(:,i)/Sumrobetall(i,m);
        end
    end
    for i=1:N
        for m=1:M
            Segi=NSegall(m)(i)/Sumrobetall(i,m);
                    Seg(m)(i)=Segi+(%eps^2)*eye(K,K);
        end
    end
