function [O,X]=cobsgen(Pinit,A,Me,Seg,C)

// This function is used to generate an observation sequence of
// length T using the given Multivariate Continuous single, M=1, C=ones(N,M)
// or a Mixture, M > 1, C=ones(N,M) HMM (Pinit,A,Seg,Me,C)
// To obtain a pre-definedHMM model, execute the function cinhmm.sci before using this function
 
// O     	: Generated observation sequence matrix (K*T)
// X     	: Generated state sequence vector (1*T)
// Pinit 	: Initial state distribution vector (1*N)
// A     	: State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*M) 	: Case of Single Gaussian
// C (N*M)	: Case of M mixtures
// M		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// i 		: state index
// m 		: mixture index

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[lhs,rhs]=argn(0);
if rhs~=5
	printf("Wrong number of input arguments in function COBSGEN ! Function call is :\n");
	printf("function [ O , X ] = cobsgenm ( Pinit , A , Seg , Me , C )\n");
	printf("Program termination\n");
	abort;
end
if lhs~=2
	printf("Wrong number of output arguments. Continue ? ( y/n )\n");
	ans=read(%io(1),1,1,'(a)');
	if ~((ans=='y')|(ans=='Y'))
		printf("Program termination\n");
		abort;
	end
end
check=%f
while ~check,
T=1000;
T=evstr(x_dialog(['HMM SIMULATION PARAMETERS- OBSERVATION SEQUENCE LENGTH';
'Please introduce the observation sequence length used to train HMM';
'You may modify directly this simulation parameter in the following box:';
'OK: continue, Cancel: abort'],table(T)));
    if T~=[],
        check=%t;
    else
        yorn();
    end
end

O1=zeros(K,T);
if ~cchkparm(Pinit,A,Me,Seg,C,O1)	// O1 is neglegted in this check
	printf("Program termination\n");
	abort;
end

[N,N]=size(A);
K=size(Me(1),1);
M=size(Me);
O=zeros(K,T);
// Calculate cumulated probability for A, Pinit and C
CumA=A;
CumPi=Pinit;
for i=1:N-1
	CumA(:,i+1)=CumA(:,i)+A(:,i+1);
	CumPi(i+1)=CumPi(i)+Pinit(i+1);
end
CumC=C;
for m=1:M-1
	CumC(:,m+1)=CumC(:,m)+C(:,m+1);
end
// choose initial state
rand('uniform');
x=rand(1);
i=1;
while x>CumPi(i), i=i+1; end
for t=1:T
	X(t)=i;
	// Choose mixture randomly
	rand('uniform');
	x=rand(1);
	m=1;
	while x>CumC(i,m), m=m+1; end
	// Calculate random observation with mixture m
	Met=Me(m)(:,i);
	U=Seg(m)(i);
	[EigVal,Q]=bdiag(U);
	rand('normal');
	y=rand(K,1);
	y=sqrt(EigVal)*y;
	O(:,t)=Q*y+Met;
	// Choose next state
	rand('uniform');
	x=rand(1);
	j=1;
	while x>CumA(i,j), j=j+1; end
	i=j;
end
X=X';




