function [LP,Pinit,A,Me,Seg,C]=creest(Pinit,A,Me,Seg,C,O,tol,IterMax)

// This function reestimates the CHMM model for many iterations on "crecursg". The conditonal probability of
// a (K) dimonsional observation is considered to be a single (M=1) or a 
// mixture (M>1) of gaussian densities with mean vectors (Me) and covariance matrices
// (Seg).
// This function takes the underflow problem into account. 
// The reestimation formulas are based on Liporace , Juang and Rabiner Algorithms.  
// L.Liporace: Maximum likelihood estimation for multivariate observations of Markov sources.
// IEEE. Trans. Inform. Theory, IT-28, pp. 729-734, 1982.  
// B. H. Juang. Maximum Likelihood Estimation for Multivariate Mixture Observations of 
// Markov Chains. IEEE Transactions on Information Theory, Vol IT-32, No. 2, March 1986.
// L.R.Rabiner et al.: A model-based connected-digit recognition system
// using either HMMs or templates, IEEE Computer Speech and Language 1986 
// L.R. Rabiner: A Tutorial on HMM and selected applications in speech recognition, IEEE 1989

// Pinit 	: Initial state distribution vector (1*N)
// A     	: State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*M) 	: Case of Single Gaussian
// C (N*M)	: Case of M mixtures
// M		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// LP		: Normalized Likelihood (log(P(O))/T
// O     	: Given observation sequence matrix (K*T)
// tol 	 	: Stop if abs((New log(likelihood) - old log(liklihood)) / old log(likelihood)) is less than tol or Iteration > IterMax. 
// IterMax 	: Maximum Iteration number

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=8,
	printf("Wrong number of input arguments in function CREEST. Function call is:\n");
	printf("[ LP , Pinit , A , Seg , Me , C ] = creest ( Pinit , A , Me , Seg , C , O , tol , IterMax)");
	printf("Program termination\n");
	abort;
end

// check of the consistency of the model parameters

if lhs~=6,
	printf("Wrong number of output arguments in function CREEST. Function call is:\n");
	printf("[ LP , Pinit , A , Seg , Me , C ] = creest ( Pinit , A , Me , Seg , C , O , tol , IterMax)");
	printf("Program termination\n");
	abort;
end

if ~cchkparm(Pinit,A,Me,Seg,C,O),	// check if the sizes of A,Pinit, Me, Seg are consitent
		printf("Program termination\n");
		abort;
end
// Me=Me(1), Seg=Seg(1) in case of a single gaussian densities
if (size(C,2)==1) then 
	[LP,Pinit,A,Me,Seg]=creestsg(Pinit,A,Me,Seg,O,tol,IterMax);
else
	[LP,Pinit,A,Me,Seg,C]=creestmg(Pinit,A,Me,Seg,C,O,tol,IterMax);
end



















