function [LP,Pinit,A,Me,Seg]=creestsg(Pinit,A,Me,Seg,O,tol,IterMax)

// Last change 03.01.98   
// This function calculates the Forward and Backward Probabilities 
// and then reestimates the CHMM model. The conditonal probability of
// a (K) dimonsional observation is considered to be a single 
// gaussian density with mean vectors (Mi) and covariance matrices
// (Segi).
// This function calls the function crecursg until the improvement of the absolute value of the relative normalized 
// log-probability (abs(LProb-LProbold)/LProbold) is less than a given tolerance tol
// or the number of iterations exceeds IterMax 
// This function takes the underflow problem into account. 
// The reestimation formulas are based on Liporace and Rabiner Algorithms.  
// L.Liporace: Maximum likelihood estimation for multivariate observations of Markov sources.
// IEEE. Trans. Inform. Theory, IT-28, pp. 729-734, 1982.  
// L.R.Rabiner et al.: A model-based connected-digit recognition system
// using either HMMs or templates, IEEE Computer Speech and Language 1986 
// L.R. Rabiner: A Tutorial on HMM and selected applications in speech recognition, IEEE 1989
   
// O     	: Given observation sequence matrix (K*T)
// Pinit 	: Initial state distribution vector (1*N)
// LP		: Normalized log of Likelihood (log(P(O|model)/T)
// A     	: Given State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me=Me(1)    	: Observation mean list Me=list(1:M)Mi
// Mi=Seg(1)	: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// M=1		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// i 		: state index
// tol 	 	: Stop if abs((New log(likelihood) - old log(liklihood)) / old log(likelihood)) is less than tol or Iteration > IterMax. 
// IterMax 	: Maximum Iteration number

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=7,
	printf("Wrong number of input arguments in function CREESTSG. Function call is:\n");
	printf("[ LP , Pinit , A , Seg , Me ] = creestsg ( Pinit , A , Me , Seg , O , tol , IterMax)");
	printf("Program termination\n");
	abort;
end
if lhs~=5,// check if number of output arguments is correct
	printf("Wrong number of output arguments. Continue ? ( y/n )\n");
	printf("If you want to continue type  y  else  n : \n");
	t=read(%io(1),1,1,'(a)');
	if (t~='y')&(t~='Y')
		printf("Program termination\n");
		abort;
	end
end
// check of the consistency of the model parameters

if ~cchkpars(Pinit,A,Me(1),Seg(1),O),	// check if the sizes of A,Pinit, Me, Seg are consitent
	printf("Program termination\n");
	abort;
end

printf("====> Requested tolerance tol (to be compared to RCNL) = %5.10f\n",tol);
printf("====> Requested Maximum iteration IterMax (if tol is not satisfied) = %d\n",IterMax);
printf("**> Iteration = 1\n");
// For single gaussian mixture M=1
Me=Me(1);Seg=Seg(1);
[LProb,Pinit,A,Me,Seg]=crecursg(Pinit,A,Me,Seg,O);
printf("-> Normalized Log of Likelihood (log(P(O|model))/T) = %5.10f\n",LProb);
LProbold=LProb;
LP=LProb;
printf("**> Iteration = 2\n");
Me=Me(1);Seg=Seg(1);
[LProb,Pinit,A,Me,Seg]=crecursg(Pinit,A,Me,Seg,O);
printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
RCNL=abs((LProb-LProbold)/LProbold);
printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
Iter=2;
LP=[LP LProb];
while ((RCNL>tol)&(Iter<IterMax))
	LProbold=LProb
	Iter=Iter+1;
	printf("**> Iteration = %d",Iter);
	Me=Me(1);Seg=Seg(1);
	[LProb,Pinit,A,Me,Seg]=crecursg(Pinit,A,Me,Seg,O);
	printf("-> Normalized Log of Likelihood (log(P(O|model))/T) = %5.10f\n",LProb);
	RCNL=abs((LProb-LProbold)/LProbold);
	printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
   	LP=[LP LProb];
end






















