function [LP,X]=cviterbi(Pinit,A,Me,Seg,C,O)

// This function (scaled Viterbi algorithm) generates, for a 
// Multivariate Single or Mixture of continuous Gaussian observation 
// sequence, the best state sequence. 
// The (log) function is used to eliminate the underflow problem. 

// O     	: Given observation sequence matrix (K*T)
// Pinit 	: Initial state distribution vector (1*N)
// A     	: Given State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// M		: Number of Gaussian observation mixtures (scalar)
// K		: Observation mean vector dimension
// N		: Number of states in HMM (scalar)
// T     	: Observation sequence length (scalar)
// C     	: Mixture gains (for a mixture of Gaussians probability density function)
// C=ones(N*1) 	: Case of Single Gaussian
// C (N*M)	:Case of M mixtures.// i 		: State index
// X		: State sequence
// P		: Likelihood P(O)
// LP		: Normalized log of Likelihood (log(P(O))

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[N,N]=size(A);
[K,T]=size(O);

X=zeros(1,T);
Phi=zeros(N,T);        // Phi : array to keep the best scores along 
                       //       a single path at time t
Psi=zeros(N,T);        // Psi : array to keep track of the best path

if (size(C,2)==1) then 
// GaussMat is a matrix (N*T) of bi(Ot)  

	[GaussMat]=gengauss(Me,Seg,O);
else
	[GaussMatm,GaussMat]=gengausm(C,Me,Seg,O);
end
//
	
for i=1:N
	for j=1:N
		if A(i,j)>0
			lA(i,j)=log(A(i,j));
		else
			lA(i,j)=-100000;  // set log(prob) to -100000 if prob=0
		end
	end
end

// Initialisation

for i=1:N
	if (GaussMat(i,1)>0),
		GaussMat(i,1)=log(GaussMat(i,1));
	else
		GaussMat(i,1)=-1000;	// set log(prob) to -1000 if prob=0
	end
end

for i=1:N
	if Pinit(i)>0
		Phi(i,1)=log(Pinit(i))+GaussMat(i,1);	
	else
		Phi(i,1)=-1000+GaussMat(i,1);	// set log(prob) to -1000 if prob=0
	end
end

// Recursion

for t = 2:T
	for i=1:N
		if (GaussMat(i,t)>0),
			GaussMat(i,t)=log(GaussMat(i,t));
		else
			GaussMat(i,t)=-1000;	// set log(prob) to -1000 if prob=0
		end
	end
	for j =1:N
		xx=Phi(:,t-1)+lA(:,j);
		[x,i]=max(xx);			
		Phi(j,t)=x+GaussMat(j,t);    
		Psi(j,t)=i;				
	end  
end
[LP,XT]=max(Phi(:,T));			
X(T)=XT;	

// Backtracking

for k=1:T-1
	t=T-k;
	X(t)=Psi(X(t+1),t+1);		
end









