function [ok] = dchkpar(Pinit,A,B)

// This function checks if the matrices Pinit, A and B are valid
// matrices for a discret HMM, i.e. checks if the sizes match,
// if the sum of probabilities is 1, if probabilities are > 0

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// O	 : Observation sequence matrix (1*T)	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length 
// ok	 : A logical variable (T or F)

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[lhs,rhs]=argn(0);

ok=%t;                  // value to be returned if everything is correct

[rPinit,cPinit]=size(Pinit);
[N,cA]=size(A);
[rB,M]=size(B);

// check of Pinit

flag=%t;
if (rPinit~=1),         // check the size of Pinit
    printf("Pinit is not a vector\n");
    ok=%f;
end
for i=1:N
    if Pinit(i)<0,        // check if all elements of Pinit are > 0
        printf("The element Pinit(%d) is < 0.\n",i);
        flag=%f;
        ok=%f;
    end
end
if flag,
    if abs(sum(Pinit)-1)>10e-10,  // check if elements of Pinit sum to 1 
                                // ( check only if all Pinit(i)>0 )
       printf("Elements of Pinit must sum to 1.");
       ok=%f;
    end
end

// Check of A

flag=%t;
if (cPinit~=N)|(cPinit~=cA),     // comparison of the size of A with the 
                                 // length of Pinit
    printf("Sizes of Pinit and A do not match\n");
    ok=%f;
end
for i=1:N
    for j=1:N
        if A(i,j)<0,                 // check if all elements of A are > 0
            printf("The element A(%d,%d) is < 0",i,j);
            flag=%f;
            ok=%f;
        end
    end
end
if flag,
    check=abs(A*ones(N,1)-1);
    for i=1:N
        if check(i)>1e-10,           // check if rows of A sum to 1 
                                     // ( check only if all(A(i,j)>0 )
            printf("Row %d of A does not sum to 1",i);
            ok=%f;
        end
    end
end

// Check of B

flag=%t;
if (rB~=N),             // comparision of the sizes of A and B
    printf("Sizes of A and B do not match\n");
    ok=%f;
end
for i=1:N
    for m=1:M
        if B(i,m)<0,             // check if all elements of A are > 0
            printf("The element B(%d,%d) is < 0",m,i);
            flag=%f;
            ok=%f;
        end
    end
end
if flag,
    for i=1:N
        check=sum(B(i,:))-1;
        if check>10e-10,    // check if rows of B sum to 1 
                            // (check only if all B(m,i)>0)
        printf("Row %d of B does not sum to 1",i);
        ok=%f;
        end
    end
end
