function [Obs,Xs]=dgenlr(Pinit,A,B)

// function to generate observation matrices for DHMM left to right model
// reestimation test. The several observations are concatenated with their
// length in between them to obtain Obs.
// The vector Tv contains the observation lengths for the several observations
  
// Pinit : Initial state distribution vector (1*N), Pinit=[1 0 0 ...] 
// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Obs	 : Observation sequences vector :the several observation sequences O are';
//	   concatenated with their length Ts in between them';
//	   size(Obs): (1,L+Ts1+Ts2+...+TsL)
// Xs	 : State sequences vector :the several state sequences X are';
//	   concatenated with their length Ts inbetween them';
//	   size(Xs): (1,L+Ts1+Ts2+...+TsL)
// O	 : Observation sequence vector (1*T)
// X	 : State sequence vector (1*T)
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=3,
    printf("Wrong number of input arguments in function DGENLR. Function call is:\n");
    printf("[Obs,Xs] = dgenlr ( Pinit , A , B )\n");
    printf("Program termination\n");
    abort;
end
if lhs~=2,
    printf("Wrong number of output arguments. Continue ? ( y/n )\n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y'),
        printf("Program termination\n");
        abort;
    end
end

// check of the consistency of the model parameters

if ~dchkpar(Pinit,A,B), // check if the sizes of A,Pinit, Me, Seg are consitent
    printf("Program termination\n");
    abort;
end

check=%f
while ~check,
L=3000;
L=evstr(x_dialog(['NUMBER OF OBSERVATION SEQUENCES (L)';
		'You have chosen Left-to-Right DHMM.';
		'You may introduce the number of the several observation sequences';
		'OK: continue, Cancel: abort'],table(L)));
    if L~=[],
        check=%t;
    else
        yorn();
    end
end
// Tv 	 : Vector contains the observation lengths for the several observation sequences
check=%f
while ~check,
Tv=50*ones(1,L);// for demostration
Tv=evstr(x_dialog(['LENGTHS OF EACH OBSERVATION SEQUENCE';
		'You have chosen Left-to-Right DHMM.';
		'You may introduce the observation sequence lengths for the several sequences';
		'Fo the demonstration example, we consider that all sequences have'; 
		'the same length='+string(Tv(1))+' ';		
		'Then the several observations are concatenated with their length in between them';
		'to obtain the observations vector Obs';
		'Example: for L=3 observation sequences, of length T=200, 400, and 300 respectively.';
		'Then  Tv=[200 400 300]';
		'In this case Obs=[200 O1 O2 .. O200 400 O1 O2 .. O400 300 O1 O2 .. O300]';
		'Oi are the discrete symbols';
		'OK: continue, Cancel: abort'],table(Tv)));
    if Tv~=[],
        check=%t;
    else
        yorn();
    end
end

[N,N]=size(A);
[N,M]=size(B);

i=1;
for l=1:L
    T=Tv(l);
    Obs(i)=T;  // observation  sequence length is written in between the observations
    Xs(i)=T;   // state sequence length is written in between the observations
    O=zeros(1,T);
    X=zeros(1,T);
// Observations and states  generation

// Calculate cummulated probabilities
CumA=A;
CumPi=Pinit;
for i=2:N
    CumA(:,i)=CumA(:,i-1)+A(:,i);
    CumPi(i)=CumPi(i-1)+Pinit(i);
end
CumB=B;
for m=2:M
    CumB(:,m)=CumB(:,m-1)+B(:,m);
end

// choose first state

x=rand(1);
i=1;
while x>CumPi(i), i=i+1; end
for t=1:T
    X(t)=i;
    // choose observation
    x=rand(1);
    m=1;
    while x>CumB(i,m), m=m+1; end
    O(t)=m;
    // choose next state
    x=rand(1);
    j=1;
    while x>CumA(i,j), j=j+1; end
    i=j;
end

//    Obs(i+1:i+T)=O;
      Obs(i:i+T-1)=O;
      Xs(i+1:i+T)=X;
//    i=i+T+1;
      i=i+T;
end





