function [distsym,distnsym]=dhmmdist(HMMA,HMMB,O)

// This function calculates the non-symmetric and symetric distances between two DHMMs 
// The models may have different N and are given as a list HMMx 
// (can be generated with the function 'dhmmlist' 

// HMMA, HMMB 		: deux lists de CHMM each of dimension 5.
// Pinita = HMMA(1)	: Initial state distribution vecor of HMMA (1*N)
// Aa     = HMMA(2)   	: State stochastic matrix of HMMA (N*N)
// Ba     = HMMA(3)     : Observation stochastic matrix of HMMA (N*M)
// LPa	  = HMMA(4)	: log likelihood of O per model a (LPa=log(P(OIHMMA))
// Coma	  = HMMA(5)	: commentary for HMMA
// N			: Number of states in HMM (scalar)
// M	 		: Number of distinct observation symbols per state
// O      		: Observation sequence (1*T)
// T      		: Observation sequence length
// dist	  		: Normalized Log likelihood ((LPa-LPb)/T)
// Pinitb, Ab, Bb and LPb are defined by HMMB in the same manner
// as in HMMA
// distnsym		: distance non-symetric between HMMA and HMMB (distnsym=(LPa-LPb)/T);
// distsym		: distance symetric between HMMA and HMMB (distsym=[dist(A,B)+dist(B,A)]/2T);

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


// check of the function call

[lhs,rhs]=argn(0);
if rhs~=3
    printf("Wrong number of input arguments in function DHMMDIST !\nFunction call is :\n");
    printf("[ dist ] = dhmmdist (HMMA , HMMB , O )");
    printf("Program termination\n");
    abort;
end
if lhs~=1
    printf("Wrong number of output variables.\nContinue? (y/n)\n");
    ans=read(%io(1),1,1,'(a)');
    if ~((ans=='y')|(ans=='Y'))
        printf("Program termination\n");
        abort;
    end
end

[Pinita,Aa,Ba,LPa,Coma]=dhmmdeli(HMMA);
[Pinitb,Ab,Bb,LPb,Comb]=dhmmdeli(HMMB);

// check of the consistency of the model parameters for the two models

if ~dchkpar(Pinita,Aa,Ba), 
    printf("Program termination\n");
    abort; 
end    
if ~dchkpar(Pinitb,Ab,Bb), 
    printf("Program termination\n");
    abort; 
end

[rO,T]=size(O);                // T : observation length

[LP1,P1]=dlikelih(Pinita,Aa,Ba,O);
[LP2,P2]=dlikelih(Pinitb,Ab,Bb,O);
distnsym=(LP1-LP2)/T;
distsym=[(LPa-LPb)+(LPb-LPa)]/(2*T);
