function [P,LP]=dlikelih(Pinit,A,B,O)

// function to calculate the probability density function that a given observation has
// been generated by the given DHMM. The underflow problem is taken into account.

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// O	 : Observation sequence matrix (1*T)	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// LP 	 : Normalized log of likelihood of HMM (log(P(O|model)/T)
// P	 : Likelihood

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=4,            // check if number of input arguments is correct
    printf("Wrong number of input arguments in function DPROB.\nFunction call is: \n");
    printf("    [ P , LP ] = dlikelih ( Pinit, A , B , O )\n");
    abort;
end
if lhs~=2,            // check if number of output arguments is correct
    printf("Wrong number of output arguments.\nContinue ? ( y/n )\n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y'),
        abort;
    end
end

// check of the consistency of the model parameters

if ~dchkpar(Pinit,A,B),
    abort;
end

[w,T]=size(O);        // T : oberservation length     
[N,N]=size(A);        // N : number of states
[N,M]=size(B);        // M : number of observations per state
s=zeros(1,T);         // s : scaling factor
Alpha=zeros(N,T);     // Alpha : array to store the forward prob.
Alphas=zeros(N,T);    // scaled Alpha

// calculation of Alpha at t=1 for i=1,..,N 

t=1;
Pvec=Pinit.*B(:,O(1,t))';
Alpha(:,t)=Pvec';
s(t)=1/sum(Alpha(:,t));
Alphas(:,t)=s(t)*Alpha(:,t);              // Rab1 (19) and (92b)
Pvec=Alphas(:,t)';

// calculation of Alpha for t=1,..,T and for i=1,..,N 

for t=2:T
    Pvec=(Pvec*A).*B(:,O(1,t))';
    Alpha(:,t)=Pvec';                            
    s(t)=1/sum(Alpha(:,t));
    Alphas(:,t)=s(t)*Alpha(:,t);          
    Pvec=Alphas(:,t)';
end
// Calculation of P(O|model)
if or(s==0) then
  LP= -%inf
  P=exp(LP);
else
  LP=-sum(log(s));
  P=exp(LP);
  LP=LP/T; // normalized log of likelihood		
end



