function [O,X]=dobsgen(Pinit,A,B)

// This function is used to generate random observation and state
// sequences of a given DHMM.

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// O	 : Observation sequence matrix (1*T)	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// X :     Complete state sequence vector (1*N)

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

[lhs,rhs]=argn(0);
if rhs~=3,            // check if number of input arguments is correct
    printf("Wrong number of input arguments in function DOBSGEN. Function call is: \n");
    printf("    [ O , X ] = dobsgen( Pinit , A , B )\n");
    abort;
end
if lhs~=2,            // check if number of output arguments is correct
    printf("Wrong number of output arguments.\n");
    printf("If you want to continue type  y  else  n : \n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y')
        abort;
    end
end

check=%f
while ~check,
T=1000;
T=evstr(x_dialog(['HMM SIMULATION PARAMETERS- OBSERVATION SEQUENCE LENGTH';
'Please introduce the observation sequence length used to train HMM';
'You may modify directly this simulation parameter in the following box:';
'OK: continue, Cancel: abort'],table(T)));
    if T~=[],
        check=%t;
    else
        yorn();
    end
end

// checks the consistency of the model parameters

if ~dchkpar(Pinit,A,B)
    abort;
end

[N,N]=size(A);
[N,M]=size(B);
O=zeros(1,T);
X=zeros(1,T);
// Calculate cummulated probabilities

CumA=A;
CumPi=Pinit;
for i=2:N
    CumA(:,i)=CumA(:,i-1)+A(:,i);
    CumPi(i)=CumPi(i-1)+Pinit(i);
end
CumB=B;
for m=2:M
    CumB(:,m)=CumB(:,m-1)+B(:,m);
end

// choose first state

x=rand(1);
i=1;
while x>CumPi(i), i=i+1; end
for t=1:T
    X(1,t)=i;
    // choose observation
    x=rand(1);
    m=1;
    while x>CumB(i,m), m=m+1; end
    O(t)=m;
    // choose next state
    x=rand(1);
    j=1;
    while x>CumA(i,j), j=j+1; end
    i=j;
end






