function [LP,Pinit,A,B]=drecur(Pinit,A,B,O)

// This function calculates the Forward and Backward Probabilities and
// then reestimates the DHMM model by the Baum algorithm. The 
// underflow problem is taken into account.
// The normalized log of likelihood LP is returned for the reestimated model
//  L.R. Rabiner : A Tutorial on HMM and selected 
// applications in speech recognition, IEEE 1989

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// O	 : Observation sequence matrix (1*T)	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// LP 	 : Normalized log of likelihood of HMM (log(P(O|model)/T)
// P	 : Likelihood

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved
     

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=4,            // check if number of input arguments is correct
    printf("Wrong number of input arguments in function DRECUR.Function call is: \n");
    printf("[ LP , Pint , A , B ] = drecur( Pinit, A , B , O )\n");
    abort;
end
if lhs~=4,            // check if number of output arguments is correct
    printf("Wrong number of output arguments.\n");
    printf("If you want to continue type  y  else  n : \n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y'),
        abort;
    end
end

// check of the consistency of the model parameters

if ~dchkpar(Pinit,A,B),
    abort;
end

[l,T]=size(O);        // T : oberservation length
[N,N]=size(A);        // N : number of states
[N,M]=size(B);        // M : number of observations per state
Alpha=zeros(N,T);     // Alpha : array to store the forward prob.
Beta=zeros(N,T);      // array to store the backward prob.
s=zeros(l,T);

// calculation of Alpha at and for t=1 for i=1,..,N 
t=1;
Alpha(:,1)=Pinit'.*B(:,O(1,1));
s(1)=1/sum(Alpha(:,1));
Alpha(:,1)=s(1)*Alpha(:,1);                    

if and(Alpha(:,1)==0),
	printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n",t);
	abort;
end

// calculation of Alpha for t=1,..,T and for i=1,..,N 

for t=2:T
    Alpha(:,t)=(A'*Alpha(:,t-1)).*B(:,O(1,t));                            
    s(t)=1/sum(Alpha(:,t));
    Alpha(:,t)=s(t)*Alpha(:,t);                
    if and(Alpha(:,t)==0),
		printf("All elements of Alpha(:,%d) are zero. This leads to a division by zero.\Program termnation\n",t);
		abort;
    end
end
                               
//	LP=-sum(log(s));
// Calculation of P(O\Lambda)
if or(s==0) then
  LP= -%inf
else
LP=-sum(log(s))/T; // normalized log of likelihood		
end

// Calculation of Beta at t=T for i=1,..,N
									
Beta(:,T)=s(T)*ones(N,1);	
// calculation of Beta for t=T-1,..,1 and for i=1,..,N 

for k=1:T-1
	t=T-k;
	Beta(:,t)=s(t)*A*(B(:,O(1,t+1)).*Beta(:,t+1));	
end
// Reestimation of the model parameters

// Reestimation of the initial states distribution (Pinit).

AB=Alpha(:,1)'.*Beta(:,1)';
Pinit = AB/sum(AB); 

// Reestimation of the State transition matrix (A)
// Calculation of the Denomiators for the reestimation

Is=ones(1,T)./s;
SumDB=(Alpha.*Beta)*Is';
SumDA=SumDB-Alpha(:,T).*Beta(:,T)/s(T);

if min(abs(SumDB))==0,
	printf("Warning : Too small values in the denominator in the reestimation of B).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end

if min(abs(SumDA))==0,
	printf("Warning : Too small values in the denominator in the reestimation of A ).\nThis leads to a division by zero.\n");
	printf("Program termination\n");
	abort;	
end           

// Reestimation of the State transition matrix (A)

for j=1:N
	A(:,j)=A(:,j).*(Alpha(:,1:T-1)*(B(j,O(2:T)).*Beta(j,2:T))')./SumDA
end


// Reestimation of the observation symbol probability matrix (B)

    for k = 1:M
        SumNBjk=zeros(N,1);
        for t=1:T
            if O(1,t)==k 
            SumNBjk=SumNBjk+Alpha(:,t).*(Beta(:,t)./s(t));
            end
        end
        B(:,k)=SumNBjk./SumDB;            
    end

  












