function [LP,A,B]=dvireest(Pinit,A,B,O,tol,IterMax)

// Function to reestimate the parameters of a DHMM until a certain stop 
// criterion is reached.  
// This function calls the function dvitrain until the improvement of the absolute value of the relative normalized 
// log-probability (abs(LProb-LProbold)/LProbold) is less than a given tolerance tol
// or the number of iterations exceeds IterMax 

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// O	 : Observation sequence matrix (1*T)	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// LP 	 : Normalized log of likelihood of HMM (log(P(O|model)/T)
// P	 : Likelihood
// tol 	 : Stop if abs((New log(likelihood) - old log(liklihood)) / old log(likelihood)) is less than tol or Iteration > IterMax. 
// IterMax : Maximum Iteration number

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

// check of the function call

[lhs,rhs]=argn(0);
if rhs~=6,            // check if number of input arguments is correct
    printf("Wrong number of input arguments in function DREEST.Function call is: \n");
    printf("[ LP , A , B ] = dvireest( Pinit , A , B , O , tol , IterMax )\n");
    abort;
end
if lhs~=3,            // check if number of output arguments is correct
    printf("Wrong number of output arguments.\n");
    printf("If you want to continue type  y  else  n : \n");
    t=read(%io(1),1,1,'(a)');
    if (t~='y')&(t~='Y')
        abort;
    end
end

if ~dchkpar(Pinit,A,B)
    abort
end
printf("====> Requested tolerance tol (to be compared to RCNL)= %5.10f\n",tol);
printf("====> Requested Maximum iteration IterMax (if tol is not satisfied) = %d\n",IterMax);
printf("**> Iteration = 1\n");
[LProb,A,B]=dvitrain(Pinit,A,B,O);
printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
LProbold=LProb;
LP=LProb;
printf("**> Iteration = 2\n");
[LProb,A,B]=dvitrain(Pinit,A,B,O);
printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
RCNL=abs((LProb-LProbold)/LProbold);
printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	
Iter=2;
LP=[LP LProb];
while (RCNL>tol&Iter<IterMax) 
    	LProbold=LProb        
	Iter=Iter+1;
	printf("**> Iteration = %d\n",Iter);
	RCNL_old=RCNL;
    	[LProb,A,B]=dvitrain(Pinit,A,B,O);
	printf("-> Normalized Log of Likelihood (log(P(O))/T) = %5.10f\n",LProb);
	RCNL=abs((LProb-LProbold)/LProbold);
	printf(">> Relative change in Normalized Log of Likelihood (RCNL=abs((New log(likelihood) - old log(liklihood)) / old log(likelihood))) = %5.10f\n",RCNL);	 
   	LP=[LP LProb];
end








