function [LP,X]=dviterbi(Pinit,A,B,O)

// This function (Viterbi algorithm) generates, for a discrete 
// observation sequence, the best state sequence.
// The (log) function is used to eliminate the underflow problem. 

// A     : State stochastic matrix (N*N)
// B     : Observation stochastic matrix (N*M)
// Pinit : Initial state distribution vecor (1*N)
// O	 : Observation sequence matrix (1*T)	 	
// M	 : Number of distinct observation symbols per state
// N	 : Number of states	
// T	 : Observation sequence length
// LP 	 : Likelihood of HMM (log(P(OImodel))
// P	 : Likelihood

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved	 	  

[lO,T]=size(O);        // T : oberservation length
[N,N]=size(A);        // N : number of states
[lB,M]=size(B);        // M : number of observations per state
LB=zeros(N,M);         // LB : log(B), if element of B=0 then LB=-1000
LA=zeros(N,N);         // LA : log(A), if element of B=0 then LB=-1000
X=zeros(1,T);
Phi=zeros(N,T);        // Phi : array to keep the best scores along 
                       //       a single path at time t
Psi=zeros(N,T);        // Psi : array to keep track of the best path


for i=1:N
    for m=1:M
        if B(i,m)>0
            LB(i,m)=log(B(i,m));
        else
            LB(i,m)=-100000;
        end
    end
end
for i=1:N
    for j=1:N
        if A(i,j)>0
            LA(i,j)=log(A(i,j));
        else
            LA(i,j)=-100000;
        end
    end
end

// Initialisation

for i=1:N
    if Pinit(i)>0
        Phi(i,1)=log(Pinit(i))+LB(i,O(1));
    else
        Phi(i,1)=-1000+LB(i,O(1));
    end
end

// Recursion

for t = 2:T
    for j =1:N 
        xx=Phi(:,t-1)+LA(:,j);
        [x,i]=max(xx);
        Phi(j,t)=x+LB(j,O(t));
        Psi(j,t)=i;
    end  
end
[LP,XT]=max(Phi(:,T));
X(T)=XT;                 

// Backtracking:

for k=1:T-1
    t=T-k;
    X(t)=Psi(X(t+1),t+1);
end



