function [GaussMatm,GaussMat]=gengausm(C,Me,Seg,O)

// This function calculates the probability density function (Likelihood) 
// of the observation vector (Ot) per state for a 
// mixture of M gaussian observations.
// GaussMatm : calculates for t=1,..,T  bim(Ot) in state i and mixture m
// GaussMatM : calculates at time t the sum on m of bim(Ot)

//Me    	: Observation mean list Me=list(1:M)Mi
//		  Mi are matrices  Mi (K*N)of mean vectors Msi
//		  K is the Observation mean vector dimension
//		  N is the number of states
//		  Msi are the Observation Mean vectors (K*1) in state i 
//		  M is the number of Gaussian observation mixtures
//Seg 		: Observation Covariance list Seg=list(1:M)Segi
//		  Segi is the Observation covarianc list Segi=list(1:N)Segsi
//		  Segsi is the Covariance Matrix (K*K) in state i 
// 		  (i=1, ..., N)
// O 		: Observation sequence matrix (K*T)
//		  T is the Observation sequence length 

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


N=size(Seg(1)); 	// Number of states in HMM
[K,T]=size(O); 		// Dimension of the observation sequence matrix
M=size(Seg);   		// Number of Gaussian observation mixtures
InvSeg=Seg;
DetSeg=zeros(N,M);	
c=(2*%pi)^(-K/2);
for m=1:M
	for i=1:N
		InvSeg(m)(i)=-0.5*inv(Seg(m)(i));
		DetSeg(i,m)=1/sqrt(det(Seg(m)(i)));
	end
end
GaussMatm=list(); // a list of dimension t of matrices (N,M), the matrices components are bjm(Ot)
GaussMat=zeros(N,T);
Cc=c*C.*DetSeg;
for m=1:M
	GaussMatm(m)=zeros(N,T);
end

for t=1:T
	for i=1:N
		for m=1:M
		  Mi=O(:,t)-Me(m)(:,i);
		  GaussMatm(m)(i,t)=Cc(i,m)*exp(Mi'*InvSeg(m)(i)*Mi);
		end
	end
end
GaussMat=zeros(N,T);
for m=1:M
	GaussMat=GaussMat+GaussMatm(m);
end
