function [GaussMat]=gengauss(Me,Seg,O)

// This function calculates the probability density function (Likelihood) 
// of the observation vector (Ot) per state for a Multivariate Single 
// gaussian observations mixture.  

// Me    	: Observation mean matrix (K,N); 
//		  K is the Observation mean vector dimension
//		  N is the number of states
// Mi=MeanOi	: Observation mean vector in state (K*1)
// Seg   	: Observation Covariance list Seg=list(1:N)
// Segi 	: Observation covarianc matrix (K*K)in state i 
//  		  (i=1, ..., N)
//O		: Observation sequence matrix (K*T)
//		  T is the Observation sequence length 
// GaussMat	: a matrix (N*T) of bi(Ot)  

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Grope E.S.I.E.E.
// All rights Reserved
    

N=size(Seg); 	// Number of states in HMM
[K,T]=size(O); 	// Dimension of the observation sequence matrix
InvSeg=Seg;
DetSeg=zeros(N,1);	
c=(2*%pi)^(-K/2);
	for i=1:N
		InvSeg(i)=-0.5*inv(Seg(i));
		DetSeg(i)=1/sqrt(det(Seg(i)));
	end
// The components of GaussMat are bj(Ot)
Cc=c*DetSeg;
GaussMat=zeros(N,T);
for t=1:T
	for i=1:N
		  Mi=O(:,t)-Me(:,i);
		  GaussMat(i,t)=Cc(i,1)*exp(Mi'*InvSeg(i)*Mi);	
	end
end
