function [LP,A,Me,Seg]=inchmmkm(N,O) 

// This function initializes a CHMM for a given observation sequence (O)
// and a given number of states (N) using vector quantization Kmean algorithm kmean.sci.
 

// N		: Number of states in HMM (scalar)
// O            : Observation sequence matrix of dimension (K*T);
// K		: Observation mean vector dimension
// T     	: Observation sequence length (scalar) 
// Pinit 	: Initial state distribution vector (1*N), Pinit=[1 0 0 ...]
// LP		: Normalized log of Likelihood of HMM (log(P(O|model)/T)
// A     	: State stochastic matrix (N*N)
// Seg   	: Observation Covariance list Seg=list(1:M)Segi
// Segi 	: Observation covarianc list Segi=list(1:N)Segsi
// Segsi	: Covariance Matrix (K*K) in state si 
// Me    	: Observation mean list Me=list(1:M)Mi
// Mi		: Matrices  Mi (K*N)of mean vectors Msi
// Msi		: Observation Mean vectors (K*1) in state si 
// M		: Number of Gaussian observation mixtures=1

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved

//check of the function call

[lhs,rhs]=argn(0);
if rhs~=2;
	printf('Wrong number of input arguments in function INCHMMKM. Function call is:\n ');
	printf('[ LP , A , Me , Seg ]=inchmmkm( N , O )\n');
	printf('Program termination\n');
	abort;
end 

if lhs~=4
	printf('Wrong number of output variables\n');
	printf('Continue? (y/n)\n');
	ans=read(%io(1),1,1,'(a)');
	if ~((ans=='y')|(ans=='Y')),
		printf('Program termination\n');
		abort;
	end 
end 

[K,T]=size(O);
Segi=list();
Seg=list();
Me=list();
Mi=zeros(K,N);
Nobs=zeros(N,1);

[X,z]=kmeans(O,N);

if X(1)~=1,
  Xold=X(1);
  Xlist1=find(X==1);
  Xlistold=find(X==Xold);
  X(Xlistold)=1;
  X(Xlist1)=Xold;
end;
for i=1:N
 SumSq=zeros(N,N);
 for t=1:T
    if X(t)==i
      SumSq=SumSq+O(:,t)*O(:,t)';
      Nobs(i)=Nobs(i)+1;
      Mi(:,i)=Mi(:,i)+O(:,t);
    end
 end 
  Mi(:,i)=Mi(:,i)/Nobs(i); 
  Segi(i)=(SumSq - Nobs(i)*(Mi(:,i)*Mi(:,i)'))/(Nobs(i)-1);
end 
Me(1)=Mi;
Seg(1)=Segi;

Aest=ones(N,N)*%eps;       // A : estimated state stochastic matrix

for t=2:T
    Aest(X(t-1),X(t))= Aest(X(t-1),X(t))+1;
end
SumA=sum(Aest,2);
Aest=inv(diag(sum(Aest,2)))*Aest;       // calculation of A
A=Aest;
Vit=5;
tol=0.000001;
Pinit=zeros(1,N);
Pinit(1)=1;
printf('>> Five iteration for further optimisation using the Viterbi algorithm\n'); 
[LP,A,Me,Seg]=cvireest(Pinit,A,Me,Seg,O,tol,Vit);







