function [c,z] = kmeans(x,nc);

// KMEANS : k-means clustering
// [c,z]= kmeans(x,nc)
//	x       - d*n samples
//	nc      - number of clusters wanted
//	c       - calculated membership vector
//	z  	- d*nc matrix with the mean for each cluster

// algorithm taken from Sing-Tze Bow, 'Pattern Recognition'

// Copyright (c) 1995 Frank Dellaert
// All rights Reserved

// Adapted to Scilab version by T. AL ANI (*) and C. Bunks ($). Thanks to Frank Dellaert for permission.
// Last change 97/08/01

// (*) e-mail: alanit@esiee.fr
// ($) e-mail: cbunks@bbn.com

[d,n] = size(x);

//------------------------------------------------------------------------
// step 1: Arbitrarily choose nc samples as the initial cluster centers
//------------------------------------------------------------------------
z= zeros(d,nc);
oldz = z;
for i=1:nc,
  dist = 0;
  while (dist==0)
    j = fix(rand()*n)+1;
    zi = x(:,j);
    [zw,w,dist] = nearest(z,zi);
  end
  z(:,i) = zi;
end
c = zeros(1,n);
D = zeros(nc,n);
index=zeros(1,n);
while 1==1
//------------------------------------------------------------------------
// step 2: distribute the patterns samples x to the chosen cluster domains
//         based on which cluster center is nearest
//------------------------------------------------------------------------

for j=1:nc,			// for every cluster
  center = z(:,j);		// get cluster center
//  if (center ~= oldz(:,j)),	// has it moved ?          //NB: Bug found and
  if sum(abs(center-oldz(:,j)))~=0,	// has it moved ?  //    Bug fixed CB
   D(j,:) = sqrdist(x,center);	// calculate the sqr distances from x to center
  end
end

oldz = z;

// find minimum
//[Dmin,index] = min(D);
for j=1:n
[Dmin,indx]=min(D(:,j));
index(j)=indx;
end

ind=find(index~=c);
moved =size(ind,2);
//moved = sum(index~=c);
//fprintf(2,'moved = //d\n',moved);
printf('moved = %d',moved);
c = index;
//------------------------------------------------------------------------
// step 3: Update the cluster centers
//         i.e. calculate the mean and minimize critsse for each cluster
//------------------------------------------------------------------------
[nr,z,v] = clusters(x,c,nc);
//e=sum(v,1);
//printf('Total variance= %5.3f',e(1));
//------------------------------------------------------------------------
// step 4: Check convergence
//------------------------------------------------------------------------
if (moved==0), break, end

//------------------------------------------------------------------------
end // while(1)












