function [HMM]=makehmm()

// Function to create one or more DHMM, CHMM or HMMAR using x_window dialog
// The given parameters :
// Number of states (N) :
// Number of distinct observation symbols per state (M) : for DHMM
// Observation dimension per state (K) : for CHMM and HMMAR
// Number of mixtures (M) : for CHMM 

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved


// *For all types of HMM :
//  Pinit 	: Initial state distribution vector (1*N)
//  A     	: State stochastic matrix (N*N)
//  N		: Number of states in HMM (scalar)

// *For DHMM : 
//  M		:  Number of distinct observation symbols per state in HMM
//  B           :  Observation stochastic matrix (N*M)

// *For CHMM :
//  Me    	: Observation mean vectors list Me=list(1:M)(K*N) of matrices  Mi (K*N), i=1,..,N 
//  Seg   	: Observation Covariance list Seg=list(1:M)(1:N)(K*K) of matrices Segi(K*K), i=1,..,N
//  Mi		: Observation mean vector in state i (K*1)
//  C     	: Mixture gains (for a mixture of Gaussians probability density function)
//  C=ones(N*1) : Case of Single Gaussian
//  C (N*M)	: Case of M mixtures
//  M		: Number of Gaussian observation mixtures (scalar)
//  K		: Observation mean vector dimension

// *For ARHMM :
// ar           : matrix of the regression coefficients in each state and for each
// output k (Nx(K*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2(n+1) for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.
// [N,K*(2(n+1)]=size(ar);		
//	     I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I
// ar(:,k) = I .................................................  I
//           I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I
// K            : Dimension of the system observation vector at time t.
// sig2		: Squared SIGMA matrix (K,N);

// HMM		: A list of generated HMMs
//		: HMM is a list of HMMs parameters just to be created, for example
//		: For 2 DHMM : Pinit=HMM(1)(1); A=HMM(1)(2); and  B=HMM(1)(3)
//		             : Pinit=HMM(2)(1); A=HMM(2)(2); and  B=HMM(2)(3)
//		: For 2 CHMM : Pinit=HMM(1)(1); A=HMM(1)(2); Me=HMM(1)(3); Seg=HMM(1)(4) and C=HMM(1)(5)
//			     : Pinit=HMM(2)(1); A=HMM(2)(2); Me=HMM(2)(3); Seg=HMM(2)(4) and C=HMM(2)(5)

// check of the function call

x_message(['CREATING HMM MODELS';
'You may create the martices for one or more HMM models';
'for a given number of states (N), observation dimension (K),';
'mixtures (M) with the x_window dialog'; 
'Information !';'All fault messages are printed in the scilab window !'; ....
'We apologise for any inconvenience.']);
[lhs,rhs]=argn(0);
if lhs~=1,			// check if number of output arguments is correct
  printf("Wrong number of output arguments.\nContinue ? ( y/n )\n");
  t=read(%io(1),1,1,'(a)');
  if (t~='y')&(t~='Y'),
    printf("Program termination\n");
    abort;
  end
end
check=%f;
while ~check	
  num=1;	
  num=x_dialog(['NUMBER OF HMM MODELS';
      'Please enter the number of HMM models';
      'then click on OK to continue, otherwise Cancel to abort'],string(num));	
  if num~=[],
    check=%t;
    num=evstr(num);
  else
    check=%f;
    yorn();	
  end
end
hmm=list();	
HMM=list();
for i=1:num
  dc=2;N=2;M=1;K=2;
  check=%f 
  while ~check,
    p=[dc N M K];
    n1=1;m1=size(p,2); 
    col='p ';
    row=' ';
    labelh=col(ones(1,m1))+string(1:m1);
    labelv=row(ones(1,n1))+string(1:n1);
    p=evstr(x_mdialog(['HMM'+string(i)+' SIMULATION PARAMETERS'; 
	' * p1 : dc  -> Choice of the HMM type';
	' dc=1 : HMM with Discret Observation Sequence (DHMM)';
	' dc=2 : HMM with Continuous Gaussian Observation Sequence (CHMM)';
	' dc=3 : HMM with Continuous Gaussian Autoregressive Observation Sequence (ARHMM)'; 
	' * p2 : N -> NUMBER OF STATES IN HMM'+string(i)';
	' * p3 : M -> NUMBER OF GAUSSIAN OBSERVATION MIXTURES IN HMM'+string(i)+' (used only for CHMM)';
	  '        M -> NUMBER OF DISCRET OBSERVATION SYMBOLS IN HMM'+string(i)+' (used only for DHMM)';
	    ' * p4 : K -> DIMENSION OF THE OBSERVATION VECTOR IN HMM'+string(i)+' (only for CHMM or ARHMM)';
	      'You may modify directly the simulation parameters in each box:';
	      'Please click on OK to continue, otherwise Cancel to abort'],labelv,labelh,string(p)));
    if p~=[],
      check=%t;
    else
      check=%f;
      yorn();
    end 
  end
  dc=p(1);N=p(2);M=p(3);K=p(4);

  if N==2, // for demonstration
    Pinit=[0.8 0.2];
  else
    Pinit=zeros(1,N);
  end
  x_message(['You may introduce next the parameters of the HMMs';
      'Please click on OK to continue']);
  // Initial probability vector : Pinit 					
  flag=%f;		
  while ~flag,
    check=%f 
    while ~check,
      Pinit=string(Pinit);
      n=1;m=N; 
      row=' ';
      col='p';
      labelh=col(ones(1,m))+string(1:m);
      labelv=row(ones(1,n))+string(1:n);
      Pinit=evstr(x_mdialog(['INITIAL PROBABILITY VECTOR (1,N) OF HMM('+string(i)+'): (Pinit)';
	  ' ';
	  'You may modify the values of Pinit of HMM '+string(i)+' ';
	  'pi<=1, sum(pi)=1 for all i';
	    'p1=initial probability of state 1,';
	    'p2=initial probability of state 2, ...';
	    'Please click on OK to continue, otherwise Cancel to abort'],labelv,labelh,Pinit));
      if Pinit==[],
	yorn();
	if N==2, // for demonstration
	  Pinit=[0.8 0.2];
	else
	  Pinit=zeros(1,N);
	end
      else
	Pinit=evstr(Pinit);
	check=%t;
      end 
    end	
    flag1=%t;
    flag2=%t;
    srPinit=abs(sum(Pinit)-1);
    for j=1:N
      if Pinit(j)<0 then	// check if all elements of Pinit are > 0
	printf("The element Pinit(%d) is < 0.",j);
	flag1=%f;
      else
	flag1=%t;
      end
    end
    if flag1,
      if srPinit>10e-10,	// check if elements of Pinit sum to 1
	printf("Elements of Pinit must sum to 1.");
	flag2=%f;
      else
	flag2=%t;
      end
      flag=flag1&flag2;
    end
    
  end

  // Stochastic matrix : A
  
  flag=%f;
  n=N;m=N;
  while ~flag,
    check=%f 
    while ~check,
      if N==1 then
	A=[1];
      elseif N==2 then
	A=[0.5 0.5;0.5 0.5];
      else
	A=zeros(N,N);
      end
      A=string(A);
      row='i=';
      col='j=';
      labelh=col(ones(1,m))+string(1:m);
      labelv=row(ones(1,n))+string(1:n);
      A=evstr(x_mdialog(['STOCHASTIC MATRIX (N,N) OF HMM('+string(i)+'): (A)';
	  ' ';
	  'You may modify the values of A of HMM '+string(i)+' ';
	  ' ';
	  'aij<=1 for i in {1,2,...,N}, j in {1,2,...,N} and sum(aij)=1, i in {1,2,...,N}, all j.';
	    'Please click on OK to continue, otherwise Cancel to abort'],labelv,labelh,A));
      if A==[],
	yorn();
	if N==1 then
	  A=[1];
	elseif N==2 then
	  A=[0.5 0.5;0.5 0.5];
	else
	  A=zeros(N,N);
	end
      else
	check=%t;
      end
    end
    flag1=%t;
    for k=1:N
      for j=1:N
	if A(k,j)<0,		// check if all elements of A are > 0
	  printf("The element A(%d,%d) is < 0",k,j);
	  flag1=%f;
	end
      end
    end
    srA=abs(sum(A,2)-1);
    fl=ones(1,N);
    if flag1 
      flag=%t;
      for k=1:N	
	if srA(k)>10e-10,	// check if rows of A sum to 1
	  printf("Row %d of A does not sum to 1",k);
	  fl(k)=0;
	end
      end
      if sum(fl)~=N then,
	flag=%f;
      end
    else
      flag=%f;
    end	
  end
  
  if dc==1 then
    flag=%f;
    n=N;m=M;
    while ~flag,
      check=%f 
      while ~check,
	if N==2&M==1, // for demonstration
	  B=[1.0 ; 1.0];
	elseif  N==2&M==2,
	  B=[0.5 0.5;0.5 0.5];
	  B=zeros(N,M);
	end
	B=string(B);
	col='k';
	row='i';
	labelh=col(ones(1,m))+string(1:m);
	labelv=row(ones(1,n))+string(1:n);
	B=evstr(x_mdialog(['OBSERVATION STOCHASTIC MATRIX (N,M) OF HMM('+string(i)+'): (B)';
	    ' ';
	    'You may modify the values of B of HMM '+string(i)+' ';
	    ' ';
	    'bik<=1 for i in {1,2,...,N}, k in {1,2,...,M} and sum(bik)=1, i in {1,2,...,N}, all k.';
	      'Please click on OK to continue, otherwise Cancel to abort'],labelv,labelh,B));
	  if B==[],
	    yorn();
	    if N==2&M==1, // for demonstration
	      B=[1.0 ; 1.0];
	    elseif  N==2&M==2,
	      B=[0.5 0.5;0.5 0.5];
	      B=zeros(N,M);
	    end
	  else
	    check=%t;	
	  end
	end
	flag1=%t;				
	for j=1:N
	  for k=1:M
	    if B(j,k)<0,					// check if all elements of B are > 0
	      printf('The element B(%d,%d) is < 0',j,k);
	      flag1=%f;
	    end
	  end
	end
	srB=abs(sum(B,2)-1);
	fl=ones(1,N);
	if flag1,
	  flag=%t;
	  for j=1:N	
	    if srB(j)>10e-10,// check if rows of B sum to 1	 
	      printf('row %d of B does not sum to 1\n');
	      fl(j)=0;
	    end
	  end
	  if sum(fl)~=N then,
	    flag=%f;
	  end
	else
	  flag=%f;
	end	
      end
      hmm=list(Pinit,A,B);
    elseif dc==2 then	

      // Covariance matrices list : Seg
      Segj=list();
      Seg=list();
      for m=1:M,	
	for j=1:N,
	  flag=%f;
	  while ~flag,	
	    check=%f
	    while ~check,	
	      if K==2, // for demonstration
		Segmj=[1.0 0.1;0.1 1.0];
	      else
		Segmj=zeros(K,K);
	      end
	      Segmj=string(Segmj)
	      col='col';
	      row='row';
	      labelh=col(ones(1,K))+string(1:K);
	      labelv=row(ones(1,K))+string(1:K);
	      // reading of Seg with x_window dialog
	      Segmj=evstr(x_mdialog(['OBSERVATION COVARIANCE MATRIX IN MIXTURE (K,K) '+string(m)+' AND STATE '+string(j)+': (Segmj)';
		  'IN HMM('+string(i)+')';
		  ' ';
		  'The submatrix of Seg for j='+string(j)+',m='+string(m)+' must be symmetric';
		    'and positive definite';
		    'Please enter Segmj then click on OK to continue, otherwise Cancel to abort'],labelv,labelh,Segmj));
	      if Segmj==[], 
		yorn();
		if K==2, // for demonstration
		  Segmj=[1 0;0 1];
		else
		  Segmj=zeros(K,K);
		end
	      else
		check=%t; 
	      end
	    end
	    flag1=%t;
	    flag2=%t;
	    if max(abs(Segmj-Segmj'))>%eps 
	      printf("The submatrix of Seg for j=%d,m=%d is not symmetric\n",j,m);
	      flag1=%f;	
	    else
	      flag1=%t;
	    end
	    if flag1,
	      if min(spec(Segmj))==%eps|min(spec(Segmj))<%eps then
		// check if submatrix is positive definite
		printf("The submatrix of Segmj for j=%d,m=%d is not positive definite\n",j,m);
		flag2=%f;				
	      else
		flag2=%t;
	      end
	    end
	    flag=flag1&flag2;	
	  end
	  Segj(j)=Segmj;	
	end 
	Seg(m)=Segj;	
      end	
      // Mean matrices list Me 
      Me=list();
      for m=1:M,
	check=%f
	while ~check,
	  if (K==2&N==2), // for demonstration
	    Mekj=[-7.5 -6.0;-2.5 -3.4];
	  else
	    Mekj=zeros(K,N);
	  end
	  Mekj=string(Mekj)
	  col='col';
	  row='row';
	  labelh=col(ones(1,K))+string(1:K);
	  labelv=row(ones(1,K))+string(1:K);
	  // reading of Seg with x_window dialog
	  Mekj=evstr(x_mdialog(['OBSERVATION MEAN MATRIX (K,N) IN MIXTURE '+string(m)+': (Mekj)';
	      'IN HMM('+string(i)+')';
	      ' ';
	      'Please enter Mekj then click on OK to continue, otherwise Cancel to abort'],labelv,labelh,Mekj));
          if Mekj==[], 
	    yorn();
	    if (K==2&N==2), // for demonstration
	      Mekj=[-7.0 -7.0;-3.0 -3.0];
	    else
	      Mekj=zeros(K,N);
	    end
	  else
	    check=%t; 
	  end
	end
	Me(m)=Mekj;
      end
      // Mixture gain matrix : C
      if (N==2&M==1), // for demonstration
	C=[1 1]';
      else
	C=zeros(N,M);
      end							
      flag=%f;
      n=N;m=M;
      while ~flag,,
	check=%f 
	while ~check,
	  C=string(C);
	  row='j=';
	  col='m=';
	  labelh=col(ones(1,m))+string(1:m);
	  labelv=row(ones(1,n))+string(1:n);
	  // reading of C with x_window dialog
	  C=evstr(x_mdialog(['MIXTURS GAIN MATRIX (N,M) IN HMM('+string(i)+') : (C)';
	      ' ';
	      'You may modify the values of C';
	      ' ';
	      'cjm<=1 for j in {1,2,...,N}, m in {1,2,...,M} and sum(cjm)=1, j in {1,2,...,N}, all m.';
		'Please click on OK to continue, otherwise Cancel to abort'],labelv,labelh,C));
	  if C==[],
	    yorn();
	    if N==2&M==1, // for demonstration
	      C=[1 1]';
	    else
	      C=zeros(N,M);
	    end	
	  else
	    check=%t;
	  end	
	end	
	flag1=%t;
	for ii=1:N
	  for mm=1:M
	    if C(ii,mm)<0,		// check if all elements of C are > 0
	      printf("The element C(%d,%d) is < 0\n",ii,mm);
	      flag1=%f;
	    end
	  end
	end
	srC=abs(sum(C,2)-1);
	fl=ones(1,N);
	if flag1,
	  flag=%t;
	  for ii=1:N
	    if srC(i)>10e-10,    // check if rows of C sum to 1
	      printf("Row %d of C does not sum to 1\n",ii);
	      fl(k)=0;
	    end
	  end
	  if sum(fl)~=N then,
	    flag=%f;
	  end
	else
	  flag=%f;
	end
      end
      hmm=list(Pinit,A,Me,Seg,C);	
    elseif dc==3 then
      check=%f;
      while ~check
	b0=x_dialog(['MODEL TYPE FOR HMM'+string(num)+':';' ';
	    'b0=0 for AR model:';' ';
	      'yk=-sum(i=1:n)[ai*y_(k-i)], a0=1';' ';
	      'b0=1 for ARX model';' ';
		'yk=-sum(i=1:n)[ai*y_(k-i)]+sum(j=0:n)[bj*y_(k-j)], a0=b0=1';
		'Where n is the number of output (a) or input (b) autoregression coefficients (n=na=nb)';
		'Please give b0 then click on OK to continue, otherwise Cancel to abort'],'0');
	if b0~=[],
	  check=%t;
	  b0=evstr(b0);
	else
	  tequest();
	end
      end
      check=%f;
      while ~check
	na=x_dialog(['NUMBER OF OUTPUT OR INPUT AUTOREGRESSIVE COEFFICIENTS (n=na=nb) FOR HMM'+string(num)+':';
	    'Please click on OK to continue, otherwise Cancel to abort'],'2');
	if na~=[],
	  check=%t;
	  na=evstr(na);
	else
	  yorn();
	end
      end	
      if b0==0 then
	Ng=K*(na+1);
      elseif b0==1 then
	Ng=K*2*(na+1);
      end
      check=%f;
      while ~check,
	n=N;m=Ng;
	col=' ';
	row='state ';
	ar=ones(n,m);
	ar=string(ar);
	labelh=col(ones(1,m))+string(1:m);
	labelv=row(ones(1,n))+string(1:n);
	ar=evstr(x_mdialog(['AUTOREGRESSIVE MATRIX (ar) OF HMM '+string(num)+'( ar'+string(num)+')';' ';
	    'ar   : matrix of the regression coefficients in each state and for each';
	    'output k (N*(K*Ng)) where Ng=n+1 for b0=0 (AR model) or Ng=2*(n+1)'; 
            'for b0=1 (ARX model). The ar(:,(k-1)*Ng+1), k=1,2,...K are always one.';' ';
	    '[N,K*(2(n+1)]=size(ar)';' ';		
	    '            I 1 a1(1)     . . .  an(1) 1 b1(1)     . . .  bn(1)  I';
            '  ar_k =    I .................................................  I';
	    '            I 1 a1(N)     . . .  an(N) 1 b1(N)     . . .  bn(N)  I';' ';
	    ' K   : Dimension of the system observation vector at time t.';' ';	
	    'Please click on OK to continue, otherwise Cancel to abort'],labelv,labelh,ar));
	if ar==[],
	  yorn();
	  ar=ones(n,m);
	else
	  ar=evstr(ar);
	  check=%t;
	end
      end
      flag=%f;
      while ~flag,
	check=%f;
	while ~check,
	  n=K;m=N;
	  col='sig2 ';
	  row='system output  ';
	  sig2=ones(n,m);                        
	  sig2=string(sig2);
	  labelh=col(ones(1,m))+string(1:m);
	  labelv=row(ones(1,n))+string(1:n);
	  sig2=evstr(x_mdialog(['SEQUARED SIGMA  MATRIX (K*N) OF HMM '+string(num)+'( sig2'+string(num)+')';' ';
	      'Please edit sig2';
	      'then click on OK to continue, otherwise Cancel to abort'],labelv,labelh,sig2));
	  if sig2==[],
	    yorn();
	    sig2=ones(K,N);
	  else
	    sig2=evstr(sig2);
	    check=%t;
	  end	
	end
	for k=1:K
	  for j=1:N
	    if sig2(k,j)<0,		
	      printf("The element sig2(%d,%d) is < 0\n",k,j);
	      flag1=%f;
	    else	
	      flag=%t;
	    end
	  end
	end	
      end	
      hmm=list(Pinit,A,ar,sig2);		
    end
    HMM(i)=hmm;		
  end
  check=%f;
  while ~check,
    d=x_message(['SAVING THE GENERATED MODEL(S)';
	'You may save in binary format the generated HMMs in a list HMM';
	'HMM=list(hmm_1,hmm_2,..., hmm_num)';
      'In DHMM case : hmm_i=list(Pinit,A,B)';
      'In CHMM case : hmm_i=list(Pinit,A,Me,Seg,C)';
	'In ARHMM case: hmm_i=list(Pinit,A,ar,sig2)';
	'i=1, 2, ..., num';' ';
	'This list may be loaded afterwards using the Scilab ''load'' command'; 
	'Would you like to save the list of generated HMMs?'; 
	'If ok click on Save';
	  'otherwise click on No save to terminate without saving.'],['Save';'No save']);
    if d==1 then
      sv_hmml(HMM);
      check=%t;
    elseif d==2 then
      test=x_message(['CAUTION : You pressed Non. Would you like realy to terminate without saving?'],['No save';'Save']);
      if test == 1
	x_message('Hope to see you again soon!');
	return;
      else
	check=%f;
      end
    end 
  end









