function [HMMslist]=training();

// This function is used as an example to show how to train a HMM.
// Training of a Discret observations HMM (DHMM)
// Training of a Continuous observations HMM (CHMM) for a single or a mixture of Gaussian observations sequence
// Training of an AutoRegressif HMM (ARHMM) for a single or a mixture of Gaussian observations sequence

// HMMslist : A list of old HMM parameters, new HMM parameters, Normalized log of Likelihood,
// used observation sequence and their corresponding state sequence:
// HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs) for DHMM
// HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs) for CHMM


// Copyright (c) 2003 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved
    

IterMax=50;	// Maximum iteration number for training
tol=0.000001;	// Stop if [(New likelihood - Old liklihood) / Old likelihood] is less than tol
// C=ones(N,M);	 Single Gaussian observations sequence
// N=number of states=2
// M=number of Gaussian mixtures
// M=1;		// Single Gaussian
// M>1		// Mixture of Gaussians
HMMslist=list();
[dc,mod,lr,a]=par1();

if (a==1)&(dc==1) then // DHMM with Baum-Welch based algorithm
  check=%f;
  while ~check,
    d=x_message(['DHMM TRAINING USING BAUM-WELCH ALGORITHM';' ';
	'You have choosen Baum-Welch based algorithm for DHMM';
	  'The HMM parameters to be estimated are:';
	  ' ';
	  'Pinit : Initial probability vector (1,N)';
	  'A     : State transition matrix (N,N)';
	  'B     : Discret observation probability matrix (N,M)';' ';
	  'Where';' ';
	  'M	: Number of discret observation symbols per state';
	  'N	: Number of states';
	  'Please click on Ok to continue or Cancel to abort.'],['Ok';'Cancel']);
      if d==1 then 
	check=%t;
      elseif d==2 then
	yorn();	
	check=%f;
      end 
    end
    s1dhmm();
    printf('>> DHMM generation\n');
    printf('>> [Pinit0,A0,B0,M,N]=dinhmm('+string(mod)+','+string(lr)+')\n');
    [Pinit0,A0,B0,M,N]=dinhmm(mod,lr);
    printf('>> Please type in scilab window the generated DHMM parameters\n');
    printf('   Pinit0, A0, B0, M and N, to visualize them, else type resume\n');
    pause;
    // Ergodic model
    if lr==1 then
      s2dhmm();
      printf('>> Observation Sequence generation. Be patient please\n');
      printf('>> [Obs,X]=dobsgen(Pinit0,A0,B0)\n');      
      [Obs,Xs]=dobsgen(Pinit0,A0,B0);
      s3bwdhmm();
      [tol,IterMax]=par2();
      printf('>> Training. Be patient please\n');
      printf('>> [LP,Pinit,A,B]=dreest(Pinit0,A0,B0,Obs,tol,IterMax)\n');
      [LP,Pinit,A,B]=dreest(Pinit0,A0,B0,Obs,tol,IterMax); 
      s4dhmm();
      HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs);
      printf('>> HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs)\n');     
      printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...9\n');      
    // Left-to-Right model 
    elseif lr==2 then
      s2dlrhmm();
      printf('>> Observation Sequence generation. Be patient please\n');
      printf('>> [Obs,Xs]=dgenlr(Pinit0,A0,B0)\n'); 
      [Obs,Xs]=dgenlr(Pinit0,A0,B0);
      s3bwdlr();
      printf('>> Training. Be patient please\n');
      printf('>> [LP,Pinit,A,B]=dlrreest(Pinit0,A0,B0,Obs,tol,IterMax)\n');
      [tol,IterMax]=par2();
      [LP,Pinit,A,B]=dlrreest(Pinit0,A0,B0,Obs,tol,IterMax);     
      s4dhmm();       
      [HMMslist]=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs);
      printf('>> HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs)\n');     
      printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...10\n');
    end

  elseif (a==2)&(dc==1)&(lr==1) then // DHMM with Hamam's simulated annealing algorithm
    check=%f;
    while ~check,
      d=x_message(['DHMM TRAINING USING SIMULATED ANNEALING HAMAM''s ALGORITHM';' ';
	  'You have choosen Hamam''s based Simulated Annealing algorithm for DHMM';
	    'The HMM parameters to be estimated are:';
	    ' ';
	    'Pinit : Initial probability vector (1,N)=[1 0 0 ...]';
	    'A     : State transition matrix (N,N)';
	    'B     : Discret observation probability matrix (N,M)';' ';
	    'Where';' ';
	    'M	: Number of discret observation symbols per state';
	    'N	: Number of states';' ';
	    'Brief description of the algorithm : Given the observation sequence O, a'; 
	    'state sequence is generated at random and its probability of generating O is'; 
	    'calculated. This is called the inital cost. Afterwards one state in the'; 
	    'sequence is changed at random and the probability that this new state'; 
	    'sequence has generated O is calculated. If the latter is bigger than the';
	      'old one, or the probability of acceptance Paccept=exp((Pnew-Pold)/Temp) is'; 
	      'big enough, the change is saved. The probability of acceptance decreases'; 
	      'with decreasing temperature so that for longer iteration only better'; 
		'solutions are accepted.'; 
		'Please click on Ok to continue or Cancel to abort.'],['Ok';'Cancel']);
      if d==1 then 
	check=%t;
      elseif d==2 then
	yorn();	
	check=%f;
      end 
    end
    s1dhmm();
    printf('>> DHMM generation\n');
    printf('>> [Pinit0,A0,B0,M,N]=dinhmm('+string(mod)+','+string(lr)+')\n');    
    [Pinit0,A0,B0,M,N]=dinhmm(mod,lr);
    Pinit=Pinit0;
    s2dhmm();
    printf('>> Observation Sequence generation. Be patient please\n');
    printf('>> [Obs,Xs]=dobsgen(Pinit0,A0,B0)\n'); 	
    [Obs,Xs]=dobsgen(Pinit0,A0,B0);
    s3sadhmm();
    [wa]=par3(N);
    [wb]=par4(N,M);
    [IterMax]=par5();
    printf('>> Training. Be patient please\n');
    printf('>> [LP,A,B]=danneal(wa,wb,N,M,Obs,IterMax)\n');
    [LP,A,B]=danneal(wa,wb,N,M,Obs,IterMax);
    s4dhmm();
    HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs);
    printf('>> HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs)\n');     
    printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...9\n');        
  elseif ((a==3)&(dc==1)&(lr==1)) then // DHMM with Viterbi based algorithm
    check=%f;
    while ~check,
      d=x_message(['DHMM TRAINING USING VITERBI ALGORITHM';' ';
	  	'You have choosen Viterbi based algorithm for DHMM';
	    'The HMM parameters to be estimated are:';
	    ' ';
	    'Pinit : Initial probability vector (1,N)=[1 0 0 ...]';
	    'A     : State transition matrix (N,N)';
	    'B     : Discret observation probability matrix (N,M)';' ';
	    'Where';' ';
	    'M	: Number of discret observation symbols per state';
	    'N	: Number of states';
	    'Please click on Ok to continue or Cancel to abort.'],['Ok';'Cancel']);
	if d==1 then 
	  check=%t;
	elseif d==2 then
	  yorn();	
	  check=%f;
	end 
      end
      s1dhmm(); 
    	printf('>> DHMM generation\n');
    	printf('>> [Pinit0,A0,B0,M,N]=dinhmm('+string(mod)+','+string(lr)+')\n'); 
      [Pinit0,A0,B0,M,N]=dinhmm(mod,lr);
      Pinit=Pinit0;
      s2dhmm();
      printf('>> Observation Sequence generation. Be patient please\n');
      printf('>> [Obs,Xs]=dobsgen(Pinit0,A0,B0)\n'); 
      [Obs,Xs]=dobsgen(Pinit0,A0,B0)
      s3vidhmm();
      [tol,IterMax]=par2()
      printf('>> Training. Be patient please\n');
      printf('>> [LP,A,B]=dvireest(Pinit0,A0,B0,Obs,tol,IterMax)\n');      
      [LP,A,B]=dvireest(Pinit0,A0,B0,Obs,tol,IterMax);
      s4dhmm();
      HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs);
      printf('>> HMMslist=list(Pinit0,A0,B0,Pinit,A,B,LP,Obs,Xs)\n');     
      printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...9\n');       
    elseif (a==1)&(dc==2) then // CHMM with Baum-Welch-Juang-Rabiner
      check=%f;
      while ~check,
				d=x_message(['CHMM TRAINING USING BAUM-WELCH-LIPORACE (for one Gaussian)';
	    	'OR RABINER-WILPON-JUANG (for a mixture of Gaussians) ALGORITHM';' ';		
	     	'You have choosen Baum-Welch-Liporace or Rabiner et al.based algorithm for CHMM';
				'The HMM parameters to be estimated are:';
				' ';
				'Pinit 	: Initial probability vector (1,N)';
				'A     	: State transition matrix (N,N)';
				'Me    	: Observation mean list Me =list(1:M) of matrices (K,N)';
				'Seg   	: Observation covariance list Seg =list(1:M)(1:N) of matrices (K,K)';' ';
				'Where';' ';
				'M	: Number of Gaussian Mixturs per state';
				'N	: Number of states';
				'K	: Observation vector dimension';' '; 
				'Please click on Ok to continue or Cancel to abort.'],['Ok';'Cancel']);
				if d==1 then 
	  			check=%t;
				elseif d==2 then
	  			yorn();	
	  			check=%f;
			end 
     end
      s1chmm();
      printf('>> CHMM generation\n');
      printf('>> [Pinit0,A0,Me0,Seg0,C0,M,K,N]=cinhmm('+string(mod)+','+string(lr)+')\n');      
      [Pinit0,A0,Me0,Seg0,C0,M,K,N]=cinhmm(mod,lr);
       Pinit=Pinit0;
       C=C0;
      // Ergodic model
      if lr==1 then
				s2chmm();
				printf('>> Observation Sequence generation. Be patient please\n');
				printf('>> [Obs,Xs]=cobsgen(Pinit0,A0,Me0,Seg0,C0)\n'); 	
				[Obs,Xs]=cobsgen(Pinit0,A0,Me0,Seg0,C0);
				s3bwchmm();
				[tol,IterMax]=par2();
				printf('>> Training. Be patient please\n');
  			printf('>> [LP,Pinit,A,Me,Seg,C]=creest(Pinit0,A0,Me0,Seg0,C0,Obs,tol,IterMax)\n'); 
				[LP,Pinit,A,Me,Seg,C]=creest(Pinit0,A0,Me0,Seg0,C0,Obs,tol,IterMax);
				s4chmm();
				HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs);
				printf('>> HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,LP,Obs,Xs)\n');     
  			printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...13\n'); 
			// Left-to-Right model
      elseif lr==2 then
				s2clrhmm();
				printf('>> Observation Sequence generation. Be patient please\n');
				printf('>> [Obs,Xs]=cgenlrb(Pinit0,A0,Me0,Seg0,C0)\n');
				[Obs,Xs]=cgenlrb(Pinit0,A0,Me0,Seg0,C0);
				s3bwclr();
				[tol,IterMax]=par2();
				printf('>> Training. Be patient please\n');
				printf('>> [LP,Pinit,A,Me,Seg,C]=clrreest(Pinit0,A0,Me0,Seg0,C0,Obs,tol,IterMax)\n');
				[LP,Pinit,A,Me,Seg,C]=clrreest(Pinit0,A0,Me0,Seg0,C0,Obs,tol,IterMax);
				s4chmm();
				printf('>> HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs)\n');     
  			printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...13\n'); 
				HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs);
      end

    elseif (a==2)&(dc==2) then // CHMM with Hamam,s simulated annealing algorithm 
      check=%f;
      while ~check,
			d=x_message(['CHMM TRAINING USING SIMULATED ANNEALING HAMAM''s ALGORITHM';' ';
	    'You have choosen Hamam''s based Simulated Annealing algorithm for CHMM';
	      'The HMM parameters to be estimated are:';
				' ';
				'Pinit 	: Initial probability vector (1,N)';
				'A     	: State transition matrix (N,N)';
				'Me    	: Observation mean list Me =list(1:M) of matrices (K,N)';
				'Seg   	: Observation covariance list Seg =list(1:M)(1:N) of matrices (K,K)';' ';
				'Where';' ';
				'M	: Number of Gaussian Mixturs per state';
				'N	: Number of states';
				'K	: Observation vector dimension';' '; 
	    	'Only the single Gaussian observation case is considered in  this algorithm';
	      'i.e. C=ones(N,1)';' '; 
	      'Brief description of the algorithm : Given the observation sequence O, a';
	      'state sequence is generated at random. With the observations that are then';
	      'assigned to each state, the mean and the variance of the sample are calculated.';
	      'With these values the probability of generating O with the guessed state sequence';
	      'is calculated. This is called the initial cost. Afterward one state in the sequence';
	      'is changed at random and the probability that this new state sequence has generated';
	      'O is calculated, after changing the mean and the variance of the changed states.'; 
	      'If the new probability is bigger than the old one, or the probability of acceptance'; 
	      'Paccept=exp((Pnew-Pold)/Temp) is big enough, the change is saved. The probability of';
	      'acceptance decreases with decreasing temperature so that for longer iteration only';
	      'better solutions are accepted.';' ';
	      'Please click on Ok to continue or Cancel to abort.'],['Ok';'Cancel']);
	 if d==1 then 
	   check=%t;
	 elseif d==2 then
	   yorn();	
	   check=%f;
	 end 
       end
       s1chmm();
       printf('>> CHMM generation\n');
       printf('>> [Pinit0,A0,Me0,Seg0,C0,M,K,N]=cinhmm('+string(mod)+','+string(lr)+')\n');
       [Pinit0,A0,Me0,Seg0,C0,M,K,N]=cinhmm(mod,lr);
       Pinit=Pinit0;
       C=C0;
       if lr==1 then
       		s2chmm();
       		printf('>> Observation Sequence generation. Be patient please\n');
       		printf('>> [Obs,Xs]=cobsgen(Pinit0,A0,Me0,Seg0,C0)\n');
       		[Obs,Xs]=cobsgen(Pinit0,A0,Me0,Seg0,C0);
       		s3sachmm();
       		[wa]=par3(N);
       		[IterMax]=par5();
       		printf('>> Training. Be patient please\n');
       		printf('>> [LP,A,Me,Seg]=canneals(wa,N,Obs,IterMax)\n');
       		[LP,A,Me,Seg]=canneals(wa,N,Obs,IterMax);
       elseif lr==2 then
					s2clrhmm();
					printf('>> Observation Sequence generation. Be patient please\n');
					printf('>> [Obs,Xs,StartSeq]=cgenlrs(Pinit,A,Me,Seg,C)\n');
					[Obs,Xs,StartSeq]=cgenlrs(Pinit0,A0,Me0,Seg0,C0);
					s3sachmm();
       		[wa]=par3(N);					
					[IterMax]=par5();
					printf('>> Training. Be patient please\n');
					printf('>> [LP,A,Me,Seg]=cannlr(wa,N,Obs,StartSeq,IterMax)\n');
					[LP,A,Me,Seg]=cannlr(wa,N,Obs,StartSeq,IterMax);       
       end 
       s4sachmm();
       printf('>> HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs)\n');     
  		 printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...13\n'); 
       HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs);
       
     elseif (a==3)&(dc==2)& (lr==1) then // CHMM ergodic model with Viterbi based algorithm
       check=%f;
       while ~check,
	 d=x_message(['CHMM ERGODIC MODEL TRAINING USING VITERBI ALGORITHM';' ';
	     'You have choosen Viterbi based algorithm for CHMM';
	       'The HMM parameters to be estimated are:';
	       ' ';
	       'Pinit 	: Initial probability vector (1,N)=[1 0 0 ...]';
	       'A     	: State transition matrix (N,N)';
	       'Me    	: Observation mean list Me =list(1:M) of matrices (K,N)';
	       'Seg   	: Observation covariance list Seg =list(1:M)(1:N) of matrices (K,K)';' ';
	       'Where';' ';
	       'M	: Number of Gaussian Mixturs per state';
	       'N	: Number of states';
	       'K	: Observation vector dimension';' '; 
	     'Only the single Gaussian observation case is considered in  this algorithm';
	       'i.e. C=ones(N,1)';' '; 
	       'Please click on Ok to continue or Cancel to abort.'],['Ok';'Cancel']);
	 if d==1 then 
	   check=%t;
	 elseif d==2 then
	   yorn();	
	   check=%f;
	 end 
       end
       s1chmm();
       printf('>> CHMM generation\n');
       printf('>> [Pinit0,A0,Me0,Seg0,C0,M,K,N]=cinhmm('+string(mod)+','+string(lr)+')\n');     
       [Pinit0,A0,Me0,Seg0,C0,M,K,N]=cinhmm(mod,lr);
       Pinit=Pinit0;
       C=C0;
       s2chmm();
       printf('>> Observation Sequence generation. Be patient please\n');
       printf('>> [Obs,Xs]=cobsgen(Pinit0,A0,Me0,Seg0,C0)\n');
       [Obs,Xs]=cobsgen(Pinit0,A0,Me0,Seg0,C0);
       s3vichmm();
       [tol,IterMax]=par2();
       printf('>> Training. Be patient please\n');
       printf('>> [LP,A,Me,Seg]=cvireest(Pinit0,A0,Me0,Seg0,Obs,tol,IterMax)\n');
       [LP,A,Me,Seg]=cvireest(Pinit0,A0,Me0,Seg0,Obs,tol,IterMax);
       s4sachmm();
       printf('>> HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs)\n');     
  		 printf('>> To obtain theses parameters please type HMMslist(i), i=1, 2, ...13\n'); 	
       HMMslist=list(Pinit0,A0,Me0,Seg0,C0,Pinit,A,Me,Seg,C,LP,Obs,Xs);
     elseif (dc==3)|((((a=2&dc==1)|a==3)&(lr==2))) then // ARHMM with Juang and Rabiner based algorithm, Hamam's algorithm left-to-right algorithm with discrete observation are not available
       
       // comming soon

       x_message(['Sorry, demonstration not available for this choice';
	   'click on ok to abort.']);
     end
      


     









