function [BestSoln,BestCost]=vq_sanne(Families,O,TotIters) 

// This function is a vector quantization simulated annealing algorithm.

// Families	: Number of states;
// O            : Observation sequence matrix of dimension (K*T)
// TotIters	: Number of temperature changes in the Simulated annealing algorithm;
// BestSoln     : Classification of observations into classes, each associated to a state of dimension (1*T);
// BestCost     : Sum of squares of the deviations in the different classes;

// Copyright (c) 1998 T. AL ANI and Y. HAMAM
// Groupe E.S.I.E.E.
// All rights Reserved
   

ExecTime=timer();  
printf('>> Simulated Annealing running for %d Temperature levels\n',TotIters);
printf('   Please be patient we will keep you informed of the remaining Time\n');

//Generate Initial Solution and its cost 

Families=Families-1;
MaxVars=ones(1,size(O,2))*Families; 
T=size(O,2); 
Chain0=T; 
Alpha=0.92; 
Beta=1.05;

acceptance=0.98
Solution=round(rand(MaxVars).*MaxVars);
Asq=O.*O;
Cost=0;
for fm=0:Families
  Sel=find(fm==Solution);
  Num=size(Sel,2);
  SumA=sum(O(:,Sel),2);
  Cost=Cost+sum(Asq(:,Sel))-SumA'*SumA/Num;
end;
Incumbent=Cost;
BestCost=Cost;
 
//adjust the initial temperature 
 
Chain=Chain0; 
Delta=0; 
Count=0; 
for Trial=1:Chain
  Pos=fix((rand(1)*T))+1; 
  SolOrg=Solution(Pos); 
  while Solution(Pos)==SolOrg  
    Solution(Pos)=fix(rand()*(Families+1)); 
  end;
  
  Cost=0;
  for fm=0:Families
    Sel=find(fm==Solution);
    Num=size(Sel,2);
    SumA=sum(O(:,Sel),2)
    Cost=Cost+sum(Asq(:,Sel))-SumA'*SumA/Num;
  end;

  if Cost > Incumbent 
    Count=Count+1; 
    Delta=Delta+(Cost-Incumbent); 
  end 
  Incumbent=Cost; 
end 

Delta=Delta/Count ;
Al = Count/Chain;
Temp=Delta/log((1-Al)/(acceptance-Al)); 
//Start simulated annealing
 
ExecTime=timer();
Chain=Chain0; 
for iter=1:TotIters 
  Count=0; 
  for Trial=1:Chain 
    Soln=Solution;
    Pos=fix((rand()*T))+1; 
    SolOrg=Solution(Pos); 
    while Solution(Pos)==SolOrg  
      Solution(Pos)=fix(rand()*(Families+1)); 
    end
    Cost=0;
    for fm=0:Families
      Sel=find(fm==Solution);
      Num=size(Sel,2);
      SumA=sum(O(:,Sel),2);
      Cost=Cost+sum(Asq(:,Sel))-SumA'*SumA/Num;
    end;


    if Cost < Incumbent 
      Count=Count+1; 
      Incumbent=Cost; 
      if Cost < BestCost  
        BestCost=Cost; 
        BestSoln=Solution; 
      end 
    elseif exp((Incumbent-Cost)/Temp) > rand(1) 
      Count=Count+1; 
      Delta=Delta+Cost-Incumbent; 
      Incumbent=Cost; 
    else  
      Solution=Soln; 
    end 
  end
  Tk_1= timer();
  accepted=Count/Chain;
      nrem = TotIters-iter;
      Trem = Tk_1*(Beta*(Beta^nrem - 1)/(Beta - 1));
      select %t
       case (Trem < 300)  then
        printf('** Execution time at Temperature level %d is  %d s. Estimated remaining Time : %d s\n',...
               iter,int(Tk_1),int(Trem));
       case (Trem >= 300 & Trem < 3600)  then
        printf('** Execution time at Temperature level %d is %d s. Estimated remaining Time : %d mn\n',...
               iter,int(Tk_1),int(Trem/60));
       else 
        printf('** Execution time at Temperature level %d is %d s. Estimated remaining Time : %d h %d mn\n',...
               iter,int(Tk_1),int(Trem/3600),int((Trem-int(Trem/3600)*3600)/60));
      end  // end of select  
      printf('>> Acceptance Rate = %6.2f Best Cost= %e\n',accepted,BestCost);
  Temp=Temp*Alpha; 
  Chain=Chain*Beta;  
end
BestSoln=BestSoln+ones(1,T);

if BestSoln(1)~=1,
  Xold=BestSoln(1);
  Xlist1=find(BestSoln==1);
  Xlistold=find(BestSoln==Xold);
  c(Xlistold)=1;
  c(Xlist1)=Xold;
end;

// Plot for two dimensional observation vectors
if size(O,1)==2,
xflag=[0];
o1=O(1,:);
o2=O(2,:);
printf('>> Plot of all observation of %d vectors\n');
plotdata(O,o1,o2,xflag);  // plot all observations
xtitle('The complete Gaussian mixture data set ('+string(T)+') of two dim. vectors','o1=O(1,:)','o2=O(2,:)');
printf('>> Please type ''resume'' to continue\n');
pause;
	c=BestSoln;
	xflag=[-1;-2;-3;-4;-5;-6];
	for i=1:N,
   		Yi=O(:,find(c==i));
		[Ki,Ti]=size(Yi);
		o1=Yi(1,:);
		o2=Yi(2,:);
		printf('>> Plot of cluster %d\n',i);
		printf('>> Number of vectors =  %d\n',Ti);
   		plotdata(O,o1,o2,xflag(i)); // plot clusters
		printf('Cluster '+string(i)+' of '+string(Ti)+' vectors\n');
		printf('>> Please type ''resume'' to continue\n');
		pause;
	end 
end
































